// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: packed.cc,v 1.4 2003/03/03 21:56:59 jewillco Exp $
//
// MPI-1 provides the capability for users to pack their own messages.  A
// stream interface is provided with OOMPI.  In the following example, a
// collection of ints, chars, and floats are packed together into a
// single message and sent.  On the receiving end, they are unpacked and
// printed.


#include <iostream>
#include "oompi.h"

using namespace std;

int
main(int argc, char *argv[])
{
  OOMPI_COMM_WORLD.Init(argc, argv);
  
  int rank = OOMPI_COMM_WORLD.Rank();
  int size = OOMPI_COMM_WORLD.Size();
  
  OOMPI_Packed pack_msg(100, OOMPI_COMM_WORLD);

  // The last process packs up a message for 3 ints, 3 chars, and 3
  // floats, then broadcasts the message to the other processes.  The
  // console process prints out the received message.

  if (rank == size - 1) {

    char c; float f;

    pack_msg.Start();
    cout << "Process " << size-1 << " broadcasting: " << endl;
    for (int i = 0; i < 3; i++) {
      c = (char) i + 'A';
      f = (float) i * 1.1;
      cout << i << " " << c << " " << f << endl;
      pack_msg << i << c << f;
    }
    pack_msg.End();
  }
  
  OOMPI_COMM_WORLD[size-1].Bcast(pack_msg);

  if (rank == 0) {
    cout << "Console process receiving: " << endl;
    for (int i = 0; i < 3; i++) {
      int j; char c; float f;
      pack_msg >> j;
      pack_msg >> c;
      pack_msg >> f;
      cout << j << " " << c << " " << f << endl;
    }
  }

  OOMPI_COMM_WORLD.Finalize();
}
