// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: user_bcast.cc,v 1.4 2003/03/03 21:57:00 jewillco Exp $
//
// This program defines a function called user_broadcast.  User_broadcast
// send the message from the last node in the communicator to every other
// node using asynchronous sends and receives.  But of course, a
// broadcast is a build in function to MPI and OOMPI, so you would never
// really do this.


#include <iostream>
#include "oompi.h"

using namespace std;

void user_bcast(int buffer[], int count, OOMPI_Intra_comm comm);

int 
main(int argc, char **argv)
{
  int rank, size;
  int msg[10];

  // Start up MPI 
  OOMPI_COMM_WORLD.Init(argc,argv);

  rank = OOMPI_COMM_WORLD.Rank();
  size = OOMPI_COMM_WORLD.Size();

  // Create an array to broadcast on the last process 
  if (rank == size - 1) {
    cout << "Broadcast: ";
    for (int i=0; i<10; i++) {
      msg[i] = i;
      cout << " " << msg[i] << " ";
    }
    cout << endl;
  }

  // Do the broadcast 
  user_bcast(msg, 10, OOMPI_COMM_WORLD);

  // If we are the console, print what we got 
  if (rank == 0) {
    cout << "process  " << rank << " got:";
    for (int i=0; i<10; i++)
      cout << " " << msg[i] << " ";
    cout << endl;
  }

  // Quit MPI 
  OOMPI_COMM_WORLD.Finalize();

  return 0;
}


void
user_bcast(int buffer[], int count, OOMPI_Intra_comm comm)
{
  int rank = comm.Rank();
  int size = comm.Size();
  
  /* The last process loops to send to everyone in the communicator. */
  /* Do a Waitall to wait for all messages to finish sending */

  if (rank == size - 1 ) {

    OOMPI_Request_array reqarray(size-1);
    
    for (int dest = 0; dest < size-1; dest++ )
      reqarray[dest] = comm[dest].Isend(buffer,count);

    reqarray.Waitall();
  } 

  /* If we are not the last process process, we should go into a receive */
  /* and wait for it to complete */

  else {
    OOMPI_Request request;
    
    request = comm[size - 1].Irecv(buffer, count);
    request.Wait();
  }
}







