// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: array_message_test.cc,v 1.4 2003/02/13 21:07:48 jsquyres Exp $
//
// OOMPI test program
// Test OOMPI_Array_message functionality
// 

#include "oompi.h"
#include "oompi_test.h"


//
// Helper functions
// Test promotion aspects
//
static inline void
Test_msg_param(OOMPI_Array_message a, void *top, MPI_Datatype realtype)
{
  if (!(a.Get_type() == realtype && a.Get_top() == top))
    Fail("Parameter message mismatch (single)");
}


static inline void
Test_msg_param(OOMPI_Array_message a, MPI_Datatype realtype, int realtag)
{
  if (!(a.Get_type() == realtype && a.Get_top() && a.Get_tag() == realtag))
    Fail("Parameter message type/tag mismatch");
}


//
// OOMPI_Array_message_test
//
void
OOMPI_Array_message_test()
{
  char char_d[100];
  short short_d[100];
  int int_d[100];
  long long_d[100];
  unsigned char uchar_d[100];
  unsigned short ushort_d[100];
  unsigned u_d[100];
  unsigned long ulong_d[100];
  float float_d[100];
  double double_d[100];
#if OOMPI_HAVE_LONG_LONG_INT
  long long int llint_d[100];
#endif
#if OOMPI_HAVE_LONG_DOUBLE
  long double ldouble_d[100];
#endif
#if OOMPI_HAVE_ANSI_COMPLEX
  float_complex fcomplex_d[100];
  double_complex dcomplex_d[100];
  long_double_complex ldcomplex_d[100];
#endif

  Testing("OOMPI_Array_message");

  // char

  Testing("OOMPI_Message::char constructors");
  Test_msg_param(char_d, char_d, MPI_CHAR);
  Test_msg_param(char_d, MPI_CHAR, OOMPI_CHAR_TAG);
  OOMPI_Array_message char_msg1(char_d);
  Midtest(char_msg1.Get_type() == MPI_CHAR,
	  "Type mismatch in message");

  Midtest(char_msg1.Get_top() == char_d,
	  "Data pointer mismatch in message");

  Midtest(char_msg1.Get_tag() == OOMPI_CHAR_TAG,
	  "Tag mismatch in message");
  Pass();

  // short

  Testing("OOMPI_Message::short constructors");
  Test_msg_param(short_d, short_d, MPI_SHORT);
  Test_msg_param(short_d, MPI_SHORT, OOMPI_SHORT_TAG);
  OOMPI_Array_message short_msg1(short_d);
  Midtest(short_msg1.Get_type() == MPI_SHORT,
	  "Type mismatch in message");

  Midtest(short_msg1.Get_top() == short_d,
	  "Data pointer mismatch in message");

  Midtest(short_msg1.Get_tag() == OOMPI_SHORT_TAG,
	  "Tag mismatch in message");
  Pass();

  // int

  Testing("OOMPI_Message::int constructors");
  Test_msg_param(int_d, int_d, MPI_INT);
  Test_msg_param(int_d, MPI_INT, OOMPI_INT_TAG);
  OOMPI_Array_message int_msg1(int_d);
  Midtest(int_msg1.Get_type() == MPI_INT,
	  "Type mismatch in message");

  Midtest(int_msg1.Get_top() == int_d,
	  "Data pointer mismatch in message");

  Midtest(int_msg1.Get_tag() == OOMPI_INT_TAG,
	  "Tag mismatch in message");
  Pass();

  // long

  Testing("OOMPI_Message::long constructors");
  Test_msg_param(long_d, long_d, MPI_LONG);
  Test_msg_param(long_d, MPI_LONG, OOMPI_LONG_TAG);
  OOMPI_Array_message long_msg1(long_d);
  Midtest(long_msg1.Get_type() == MPI_LONG,
	  "Type mismatch in message");

  Midtest(long_msg1.Get_top() == long_d,
	  "Data pointer mismatch in message");

  Midtest(long_msg1.Get_tag() == OOMPI_LONG_TAG,
	  "Tag mismatch in message");
  Pass();

  // unsigned char

  Testing("OOMPI_Message::unsigned char constructors");
  Test_msg_param(uchar_d, uchar_d, MPI_UNSIGNED_CHAR);
  Test_msg_param(uchar_d, MPI_UNSIGNED_CHAR, OOMPI_UNSIGNED_CHAR_TAG);
  OOMPI_Array_message uchar_msg1(uchar_d);
  Midtest(uchar_msg1.Get_type() == MPI_UNSIGNED_CHAR,
	  "Type mismatch in message");
  
  Midtest(uchar_msg1.Get_top() == uchar_d,
	  "Data pointer mismatch in message");

  Midtest(uchar_msg1.Get_tag() == OOMPI_UNSIGNED_CHAR_TAG,
	  "Tag mismatch in message");
  Pass();

  // unsigned_short

  Testing("OOMPI_Message::unsigned short constructors");
  Test_msg_param(ushort_d, ushort_d, MPI_UNSIGNED_SHORT);
  Test_msg_param(ushort_d, MPI_UNSIGNED_SHORT, 
		 OOMPI_UNSIGNED_SHORT_TAG);
  OOMPI_Array_message ushort_msg1(ushort_d);
  Midtest(ushort_msg1.Get_type() == MPI_UNSIGNED_SHORT,
	  "Type mismatch in message");
  
  Midtest(ushort_msg1.Get_top() == ushort_d,
	  "Data pointer mismatch in message");

  Midtest(ushort_msg1.Get_tag() == OOMPI_UNSIGNED_SHORT_TAG,
	  "Tag mismatch in message");
  Pass();

  // unsigned

  Testing("OOMPI_Message::unsigned constructors");
  Test_msg_param(u_d, u_d, MPI_UNSIGNED);
  Test_msg_param(u_d, MPI_UNSIGNED, OOMPI_UNSIGNED_TAG);
  OOMPI_Array_message u_msg1(u_d);
  Midtest(u_msg1.Get_type() == MPI_UNSIGNED,
	  "Type mismatch in message");

  Midtest(u_msg1.Get_top() == u_d,
	  "Data pointer mismatch in message");

  Midtest(u_msg1.Get_tag() == OOMPI_UNSIGNED_TAG,
	  "Tag mismatch in message");
  Pass();

  // unsigned long

  Testing("OOMPI_Message::unsigned long constructors");
  Test_msg_param(ulong_d, ulong_d, MPI_UNSIGNED_LONG);
  Test_msg_param(ulong_d, MPI_UNSIGNED_LONG, OOMPI_UNSIGNED_LONG_TAG);
  OOMPI_Array_message ulong_msg1(ulong_d);
  Midtest(ulong_msg1.Get_type() == MPI_UNSIGNED_LONG,
	  "Type mismatch in message");

  Midtest(ulong_msg1.Get_top() == ulong_d,
	  "Data pointer mismatch in message");

  Midtest(ulong_msg1.Get_tag() == OOMPI_UNSIGNED_LONG_TAG,
	  "Tag mismatch in message");
  Pass();

  // float

  Testing("OOMPI_Message::float constructors");
  Test_msg_param(float_d, float_d, MPI_FLOAT);
  Test_msg_param(float_d, MPI_FLOAT, OOMPI_FLOAT_TAG);
  OOMPI_Array_message float_msg1(float_d);
  Midtest(float_msg1.Get_type() == MPI_FLOAT,
	  "Type mismatch in message");

  Midtest(float_msg1.Get_top() == float_d,
	  "Data pointer mismatch in message");

  Midtest(float_msg1.Get_tag() == OOMPI_FLOAT_TAG,
	  "Tag mismatch in message");
  Pass();

  // double

  Testing("OOMPI_Message::double constructors");
  Test_msg_param(double_d, double_d, MPI_DOUBLE);
  Test_msg_param(double_d, MPI_DOUBLE, OOMPI_DOUBLE_TAG);
  OOMPI_Array_message double_msg1(double_d);
  Midtest(double_msg1.Get_type() == MPI_DOUBLE,
	  "Type mismatch in message");
  
  Midtest(double_msg1.Get_top() == double_d,
	  "Data pointer mismatch in message");
  
  Midtest(double_msg1.Get_tag() == OOMPI_DOUBLE_TAG,
	  "Tag mismatch in message");
  Pass();

#if OOMPI_HAVE_LONG_LONG_INT
  // long long int

  Testing("OOMPI_Message::long long int constructors");
  Test_msg_param(llint_d, llint_d, MPI_LONG_LONG_INT);
  Test_msg_param(llint_d, MPI_LONG_LONG_INT, OOMPI_LONG_LONG_INT_TAG);
  OOMPI_Array_message llint_msg1(llint_d);
  Midtest(llint_msg1.Get_type() == MPI_LONG_LONG_INT,
	  "Type mismatch in message");
  
  Midtest(llint_msg1.Get_top() == llint_d,
	  "Data pointer mismatch in message");
  
  Midtest(llint_msg1.Get_tag() == OOMPI_LONG_LONG_INT_TAG,
	  "Tag mismatch in message");
  Pass();
#endif

#if OOMPI_HAVE_LONG_DOUBLE
  // long double

  Testing("OOMPI_Message::long double constructors");
  Test_msg_param(ldouble_d, ldouble_d, MPI_LONG_DOUBLE);
  Test_msg_param(ldouble_d, MPI_LONG_DOUBLE, OOMPI_LONG_DOUBLE_TAG);
  OOMPI_Array_message ldouble_msg1(ldouble_d);
  Midtest(ldouble_msg1.Get_type() == MPI_LONG_DOUBLE,
	  "Type mismatch in message");

  Midtest(ldouble_msg1.Get_top() == ldouble_d,
	  "Data pointer mismatch in message");

  Midtest(ldouble_msg1.Get_tag() == OOMPI_LDOUBLE_TAG,
	  "Tag mismatch in message");
  Pass();
#endif

#if OOMPI_HAVE_ANSI_COMPLEX
  // float complex

  Testing("OOMPI_Message::float_complex constructors");
  Test_msg_param(fcomplex_d, fcomplex_d, MPI_FLOAT_COMPLEX);
  Test_msg_param(fcomplex_d, MPI_FLOAT_COMPLEX, 
		 OOMPI_FLOAT_COMPLEX_TAG);
  OOMPI_Array_message fcomplex_msg1(fcomplex_d);
  Midtest(fcomplex_msg1.Get_type() == MPI_FLOAT_COMPLEX,
	  "Type mismatch in message");
  
  Midtest(fcomplex_msg1.Get_top() == fcomplex_d,
	  "Data pointer mismatch in message");
  
  Midtest(fcomplex_msg1.Get_tag() == OOMPI_FLOAT_COMPLEX_TAG,
	  "Tag mismatch in message");
  Pass();

  // double complex

  Testing("OOMPI_Message::double complex constructors");
  Test_msg_param(dcomplex_d, dcomplex_d, MPI_DOUBLE_COMPLEX);
  Test_msg_param(dcomplex_d, MPI_DOUBLE_COMPLEX, 
		 OOMPI_DOUBLE_COMPLEX_TAG);
  OOMPI_Array_message dcomplex_msg1(dcomplex_d);
  Midtest(dcomplex_msg1.Get_type() == MPI_DOUBLE_COMPLEX,
	  "Type mismatch in message");

  Midtest(dcomplex_msg1.Get_top() == dcomplex_d,
	  "Data pointer mismatch in message");

  Midtest(dcomplex_msg1.Get_tag() == OOMPI_DOUBLE_COMPLEX_TAG,
	  "Tag mismatch in message");
  Pass();

  // long double complex

  Testing("OOMPI_Message::long double complex constructors");
  Test_msg_param(ldcomplex_d, ldcomplex_d, MPI_LONG_DOUBLE_COMPLEX);
  Test_msg_param(ldcomplex_d, MPI_LONG_DOUBLE_COMPLEX, 
		 OOMPI_LONG_DOUBLE_COMPLEX_TAG);
  OOMPI_Array_message ldcomplex_msg1(ldcomplex_d);
  Midtest(ldcomplex_msg1.Get_type() == MPI_LONG_DOUBLE_COMPLEX,
	  "Type mismatch in message");

  Midtest(ldcomplex_msg1.Get_top() == ldcomplex_d,
	  "Data pointer mismatch in message");

  Midtest(ldcomplex_msg1.Get_tag() == OOMPI_LONG_DOUBLE_COMPLEX_TAG,
	  "Tag mismatch in message");
  Pass();
#endif

  Pass();
}

