// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: cart_comm_test.cc,v 1.7 2003/02/13 21:07:48 jsquyres Exp $
//
// OOMPI test program
// Test OOMPI_Cart_comm functionality
// 

#include "oompi.h"
#include "oompi_test.h"


//
// OOMPI_Cart_comm_test
//
void
OOMPI_Cart_comm_test()
{
  Testing("OOMPI_Cart_comm");

  Testing("OOMPI_Cart_comm::Constructors");
  
  MPI_Comm mpi_cart_comm(0);
  int ndims = 4, dims[] = { 1, 1, 1, comm_size };
  int int_periods[] = { 0, 0, 0, 0 };
  bool bool_periods[] = { false, false, false, false };
  MPI_Cart_create(MPI_COMM_WORLD, ndims, dims, int_periods, 0, &mpi_cart_comm);
  
  Testing("OOMPI_Cart_comm::Default Constructor");
  OOMPI_Cart_comm cart_comm0;
  MPI_Comm mcomm;
  mcomm = cart_comm0.Get_mpi();
  Test(mcomm == MPI_COMM_NULL);

  {
  Testing("OOMPI_Cart_comm::MPI Constructor");
  OOMPI_Cart_comm cart_comm1(mpi_cart_comm);
  Midtest(cart_comm1.Size() == comm_size);
  Test(cart_comm1.Topo_test() == OOMPI_CART);

  Testing("OOMPI_Cart_comm::Copy Constructor");
  OOMPI_Cart_comm cart_comm2(cart_comm1);
  Midtest(cart_comm2.Dim_get() == ndims);
  Midtest(cart_comm2.Size() == comm_size);
  Midtest(cart_comm2.Topo_test() == OOMPI_CART);
  Pass();

  Testing("OOMPI_Cart_comm::Assignment Operator");
  OOMPI_Cart_comm cart_comm3 = cart_comm1;
  Midtest(cart_comm3.Dim_get() == ndims);
  Midtest(cart_comm3.Size() == comm_size);
  Midtest(cart_comm3.Topo_test() == OOMPI_CART);
  Pass();

  Testing("OOMPI_Cart_comm::Typical Constructor");
  OOMPI_Cart_comm cart_comm4(OOMPI_COMM_WORLD, ndims, dims, bool_periods);
  OOMPI_Cart_comm cart_comm45(OOMPI_COMM_WORLD, ndims, dims, false);
  
  Midtest(cart_comm4.Dim_get() == ndims);
  Midtest(cart_comm4.Size() == comm_size);
  Midtest(cart_comm4.Topo_test() == OOMPI_CART);
  Midtest(cart_comm45.Compare(cart_comm4) == OOMPI_CONGRUENT);
  Pass();

  Pass(); // Constructors

  Testing("OOMPI_Cart_comm::Dim_get()");
  Test(cart_comm1.Dim_get() == ndims);

  Testing("OOMPI_Cart_comm::Dup()");
  OOMPI_Cart_comm cart_comm5;
  cart_comm5 = cart_comm1.Dup();
  Midtest(cart_comm5.Dim_get() == ndims);
  Midtest(cart_comm5.Size() == comm_size);
  Midtest(cart_comm5.Topo_test() == OOMPI_CART);
  Pass();

  Testing("OOMPI_Cart_comm::Compare()");
  Midtest(cart_comm4.Compare(cart_comm1) == OOMPI_CONGRUENT);
  Pass();

  Testing("OOMPI_Cart_comm::Sub()");
  bool keep[] = { false, true, true, false };
  OOMPI_Cart_comm cart_comm6 = cart_comm5.Sub(keep);
  Midtest(cart_comm6.Dim_get() == 2, "Sub test 1");

  if (flags[SKIP_CRAY1104])
    Pass("PASS (last test skipped/Cray 1.1.0.4)");
  else {
    OOMPI_Cart_comm cart_comm65 = cart_comm5.Sub(false, true, true, false);
    Test(cart_comm65 == cart_comm6, "Sub test 2");
  }

  int Xdims[4], Xcoords[4];
  bool Xperiods[4];
  Testing("OOMPI_Cart_comm::Get()");

  cart_comm6.Get(2, Xdims, Xperiods, Xcoords);

  Midtest(Xdims[0] == 1 && Xdims[1] == 1, "Get test 1");
  Midtest(Xcoords[0] == 0 && Xcoords[1] == 0, "Get test 2");

  cart_comm6.Get(Xdims, Xperiods);
  Midtest(Xdims[0] == 1 && Xdims[1] == 1, "Get test 3");

  if (flags[SKIP_IBM21014])
    Pass("PASS (last test skipped/IBM 2.1.0.14)");
  else if (flags[SKIP_IBM21015])
    Pass("PASS (last test skipped/IBM 2.1.0.15)");
  else if (flags[SKIP_IBM21016])
    Pass("PASS (last test skipped/IBM 2.1.0.16)");
  else if (flags[SKIP_IBM21017])
    Pass("PASS (last test skipped/IBM 2.1.0.17)");
  else if (flags[SKIP_IBM21018])
    Pass("PASS (last test skipped/IBM 2.1.0.18)");
  else if (flags[SKIP_IBM23012])
    Pass("PASS (last test skipped/IBM 2.3.0.12)");
  else
    Test(Xperiods[0] == false && Xperiods[1] == false, "Get test 4");
  
  Testing("OOMPI_Cart_comm::operator()()");
  OOMPI_Port A,B;
  A = cart_comm1(0,0,0,0);
  B = cart_comm1(0,0,0,1);
  int rank_coords0[] = {0,0,0,0};
  int rank_coords1[] = {0,0,0,1};
  Test (A.Rank() == 0 && B.Rank() == 1
	&& cart_comm1(rank_coords0).Rank() == 0 
	&& cart_comm1(rank_coords1).Rank() == 1);


  Testing("OOMPI_Cart_comm::Rank()");
  Test (cart_comm1.Rank(0,0,0,0) == 0 &&
	   cart_comm1.Rank(0,0,0,1) == 1 &&
	   cart_comm1.Rank(rank_coords0) == 0 &&
	   cart_comm1.Rank(rank_coords1) == 1);

  Testing("OOMPI_Cart_comm::Coords()");

  cart_comm1.Coords(cart_comm1.Rank(0,0,0,1), ndims, Xdims);
  Midtest(Xdims[0] == 0 &&
	    Xdims[1] == 0 &&
	    Xdims[2] == 0 &&
	    Xdims[3] == 1);
  
  cart_comm1.Coords(cart_comm1.Rank(0,0,0,0), Xdims);
  Midtest(Xdims[0] == 0 &&
	    Xdims[1] == 0 &&
	    Xdims[2] == 0 &&
	    Xdims[3] == 0);

  cart_comm1.Coords(Xdims);
  if (cart_comm1.Rank() == 1)
    if (Xdims[0] != 0 ||
	 Xdims[1] != 0 ||
	 Xdims[2] != 0 ||
	 Xdims[3] != 1)
      Fail();
  Sync();

  int *coord_ptr;
  coord_ptr = cart_comm1.Coords(cart_comm1.Rank(0,0,0,1), ndims);
  Midtest(coord_ptr[0] == 0 &&
	    coord_ptr[1] == 0 &&
	    coord_ptr[2] == 0 &&
	    coord_ptr[3] == 1);
  
  delete [] coord_ptr;
  
  coord_ptr = cart_comm1.Coords(cart_comm1.Rank(0,0,0,0));
  if (coord_ptr[0] != 0 ||
       coord_ptr[1] != 0 ||
       coord_ptr[2] != 0 ||
       coord_ptr[3] != 0)
    Fail();
  delete [] coord_ptr;

  coord_ptr = cart_comm1.Coords();
  if (cart_comm1.Rank() == 1 &&
       (coord_ptr[0] != 0 ||
	coord_ptr[1] != 0 ||
	coord_ptr[2] != 0 ||
	coord_ptr[3] != 1))
    Fail();
  delete [] coord_ptr;

  Pass();
 
  Testing("OOMPI_Cart_comm::Shift()");
  if (cart_comm1.Rank() == 1 && cart_comm1.Shift(3,-1) != 0)
    Fail();
  if (cart_comm1.Rank() == 0 && cart_comm1.Shift(3,1) != 1)
    Fail();
  Pass();

  Testing("OOMPI_Cart_comm::Test_inter");
  Test(cart_comm1.Test_inter() == false);

  Testing("OOMPI_Intra_comm::Topo_test()");
  Test(cart_comm1.Topo_test() == OOMPI_CART);

  Pass();  
  }

  MPI_Comm_free(&mpi_cart_comm);
}
