// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: comm_world_test.cc,v 1.15 2003/02/13 21:07:48 jsquyres Exp $
//
// OOMPI test program
// Test OOMPI_Comm_world functionality
// 

#include <iostream>

#include "oompi.h"
#include "oompi_test.h"


//
// OOMPI_Comm_world_test
//
void
OOMPI_Comm_world_test()
{
  int i;
  bool skip = false;
  bool pass = false;

  Testing("OOMPI_Comm_world");

  Testing("OOMPI_Comm_world::Init() (already done)");
  Pass();

  Testing("OOMPI_Comm_world::Finalize() (assumedly)");
  Pass();

  Testing("OOMPI_Comm_world::Test_inter()");
  Test(!OOMPI_COMM_WORLD.Test_inter());

  // Test stuff in OOMPI_Error_table
  // Make sure that we can exception properly
  
  Testing("OOMPI_Comm_world::OOMPI_Error_table class");
  Testing("OOMPI_Error_table::Set_error_action()");
  {
    OOMPI_COMM_WORLD.Set_error_action(OOMPI_ERRORS_ARE_FATAL);
    OOMPI_COMM_WORLD.Set_error_action(OOMPI_ERRORS_EXCEPTION);
    OOMPI_COMM_WORLD.Set_error_action(OOMPI_ERRORS_RETURN);
  }
  Pass();

  Testing("OOMPI_Error_table::Get_error_action()");
  {
    OOMPI_COMM_WORLD.Set_error_action(OOMPI_ERRORS_ARE_FATAL);
    Midtest(OOMPI_COMM_WORLD.Get_error_action() == OOMPI_ERRORS_ARE_FATAL);
    OOMPI_COMM_WORLD.Set_error_action(OOMPI_ERRORS_EXCEPTION);
    Midtest(OOMPI_COMM_WORLD.Get_error_action() == OOMPI_ERRORS_EXCEPTION);
    OOMPI_COMM_WORLD.Set_error_action(OOMPI_ERRORS_RETURN);
    Midtest(OOMPI_COMM_WORLD.Get_error_action() == OOMPI_ERRORS_RETURN);
  }
  Pass();

#if OOMPI_HAVE_EXCEPTIONS
  Testing("OOMPI_Error_table::ERRORS_EXCEPTION");
#else
  Testing("OOMPI_Error_table::ERRORS_EXCEPTION (err msgs normal)");
#endif
  {
    OOMPI_COMM_WORLD.Set_error_action(OOMPI_ERRORS_EXCEPTION);
#if OOMPI_HAVE_EXCEPTIONS
    try {
#if 0
      // There isn't a good [portable] way to check for the SGI MPI
      // implementation.  :-(
      if (getenv("MPI_CHECK_ARGS") == 0) {
	if (my_rank == 0) {
	  std::cout << std::endl << std::endl 
		    << "The OOMPI test suite depends on the MPI_CHECK_ARGS environment" << std::endl
	    << "variable being set to \"1\" *before* mpirun is invoked for successful" << std::endl
	    << "tests of exception handling.  These tests will now be skipped since" << std::endl
	    << "MPI_CHECK_ARGS is not currently set.  Set the MPI_CHECK_ARGS variable and" << std::endl
	    << "re-run the OOMPI test suite." << std::endl << std::endl;
	}
	skip = true;
      } else {
	MPI_Comm comm;
	MPI_Comm_dup(MPI_COMM_NULL, &comm);
      }
#elif LAM_MPI
      // Convenient: we know that LAM_MPI is #define'd to be 1
      // (because we own LAM! :-)

      // LAM will definitely fail on a negative tag
      MPI_Send(&i, -1, MPI_DATATYPE_NULL, -my_rank, -201, MPI_COMM_NULL);
#else
      // All others should fail with an MPI_DATATYPE_NULL
      MPI_Send(&i, -1, MPI_DATATYPE_NULL, -5000, 201, MPI_COMM_NULL);
#endif
    }
    catch (OOMPI_Error err) {
      pass = true;
    }
#else
    // We don't have exceptions, fake it
    MPI_Send(&i, 1, MPI_DATATYPE_NULL, my_rank, 201, MPI_COMM_WORLD);
    if (OOMPI_errno != OOMPI_SUCCESS)
      pass = true;
#endif
    if (skip)
      Done("Skipped (MPI_CHECK_ARGS not set)");
    else if (!pass)
      Fail("Exception not caught");
    else
      Pass();
  }

  Testing("OOMPI_Error_table::ERRORS_ARE_FATAL");
  Done("Cannot test");

  Testing("OOMPI_Error_table::ERRORS_RETURN");
  Done("Cannot test");

  // Reset error handler to ERRORS_ARE_FATAL for later tests
  OOMPI_COMM_WORLD.Set_error_action(OOMPI_ERRORS_ARE_FATAL);

  Pass(); // OOMPI_Error_table

  // Test stuff in OOMPI_Comm (abstract base class)
  // Attributes

  Testing("OOMPI_Comm_world::OOMPI_Comm base class");
  Testing("OOMPI_Comm::Is_null()");
  Postpone("OOMPI_Intra_comm");

  Testing("OOMPI_Comm::Compare()");
  Postpone("OOMPI_Intra_comm");

  Testing("OOMPI_Comm::operator==()");
  Postpone("OOMPI_Intra_comm");

  Testing("OOMPI_Comm::operator!=()");
  Postpone("OOMPI_Intra_comm");

  Testing("OOMPI_Comm::Group()");
  Postpone("OOMPI_Intra_comm");

  Testing("OOMPI_Comm::Rank()");
  Test(OOMPI_COMM_WORLD.Rank() == my_rank);

  Testing("OOMPI_Comm::Size()");
  Test(OOMPI_COMM_WORLD.Size() == comm_size);

  Testing("OOMPI_Comm::Pack_size()");
  Postpone("OOMPI_Message");

  Testing("OOMPI_Comm::Test_inter()");
  Test(!OOMPI_COMM_WORLD.Test_inter());

  Testing("OOMPI_Comm::Initialized()");
  Test(OOMPI_COMM_WORLD.Initialized() == 1);

  Testing("OOMPI_Comm::Abort()");
  Done("Cannot test");

  Testing("OOMPI_Comm::operator[]");
  Postpone("OOMPI_Port");

  Testing("OOMPI_Comm::Sendrecv()");
  Postpone("OOMPI_Message");

  Testing("OOMPI_Comm::Sendrecv_replace()");
  Postpone("OOMPI_Message");

  Pass(); // OOMPI_Comm_world::OOMPI_Comm

  // Big 4 tests
  // Default constructor

  Testing("OOMPI_Comm_world::Default constructor");
  OOMPI_COMM_WORLD.Set_error_action(OOMPI_ERRORS_EXCEPTION);
#if OOMPI_HAVE_EXCEPTIONS
  bool caught= false;
  try {
    OOMPI_Comm_world comm1;
  } 
  catch (OOMPI_Error err) {
    caught= true;
  }

  Test(caught && OOMPI_errno == OOMPI_ERR_COMM); 
#else
  OOMPI_Comm_world comm1;
  Test(OOMPI_errno == OOMPI_ERR_COMM && comm1.Is_null());
#endif

  // Copy constructor

  Testing("OOMPI_Comm_world::Copy constructor");
#if OOMPI_HAVE_EXCEPTIONS
  caught= false;
  try {
    OOMPI_Comm_world comm2(OOMPI_COMM_WORLD);
  }
  catch (OOMPI_Error err) {
    caught= true;
  }

  Test(caught && OOMPI_errno == OOMPI_ERR_COMM); 
#else
  OOMPI_Comm_world comm2(OOMPI_COMM_WORLD);
  Test(OOMPI_errno == OOMPI_ERR_COMM && comm2.Is_null());
#endif

  // Assignment operator

  Testing("OOMPI_Comm_world::operator=()");
#if OOMPI_HAVE_EXCEPTIONS
  caught= false;
  try {
    OOMPI_COMM_WORLD= OOMPI_COMM_WORLD;
  }
  catch (OOMPI_Error err) {
    caught= true;
  }
  Test(caught && OOMPI_errno == OOMPI_ERR_COMM);
#else
  OOMPI_COMM_WORLD = OOMPI_COMM_WORLD;
  Test(OOMPI_errno == OOMPI_ERR_COMM);
#endif

  // Destructor

  Testing("OOMPI_Comm_world::Destructor");
#if OOMPI_HAVE_EXCEPTIONS
  caught= false;
  for (i= 0; i < 1; i++) {
    try {
      OOMPI_Comm_world comm3;
    }
    catch (OOMPI_Error err) {
      // Do nothing;
    }
  }
  Test(OOMPI_errno == OOMPI_ERR_COMM && 
       OOMPI_COMM_WORLD.Get_mpi() == MPI_COMM_WORLD);
#else
  for (i= 0; i < 1; i++) {
    OOMPI_Comm_world comm3;
  }
  Test(OOMPI_errno == OOMPI_ERR_COMM && 
       OOMPI_COMM_WORLD.Get_mpi() == MPI_COMM_WORLD);
#endif

  // Finalized

  Testing("OOMPI_Comm_world::Finalized()");
  Test(!OOMPI_COMM_WORLD.Finalized());

  OOMPI_COMM_WORLD.Set_error_action(OOMPI_ERRORS_ARE_FATAL);

  Pass(); // OOMPI_Comm_world
}

