// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: datatype_test.cc,v 1.6 2003/02/13 21:07:48 jsquyres Exp $
//
// OOMPI test program
// Test OOMPI_Datatype functionality
// 

#include "oompi.h"
#include "oompi_test.h"


//
// User object
//
class My_object : public OOMPI_User_type {
public:
  My_object();
  inline OOMPI_Datatype &Get_type(void) { return type; };

private:
  static OOMPI_Datatype type;
  int a, b, c[10];
  char d[6], e[20];
  double f;
};

My_object::My_object() 
  : OOMPI_User_type(type, this, 202)
{
  bool passed = false;
  
  Testing("OOMPI_Datatype::Built()");
  if (!type.Built()) {
    passed = true;
    Pass();
    
    Testing("OOMPI_Datatype::Struct_start()");
    type.Struct_start(this);
    Pass();
    
    Testing("OOMPI_Datatype::operator<<()");
    type << a << b;
    Pass();
    
    Testing("OOMPI_Datatype::Entry()");
    type.Entry(c, 10);
    type.Entry(d, 6);
    type.Entry(e, 20);
    type.Entry(f);
    Pass();
    
    Testing("OOMPI_Datatype::Struct_end()");
    type.Struct_end();
    Pass();
  }
  if (!passed)
    Fail();
}

OOMPI_Datatype My_object::type;



//
// OOMPI_Datatype_test
//
void
OOMPI_Datatype_test()
{
  MPI_Aint ext, ext1, ext2;
  int size, size1, size2;
  MPI_Aint low, low1, low2;
  MPI_Aint hi, hi1, hi2;
  MPI_Datatype mpi_d1, mpi_d2;

  Testing("OOMPI_Datatype");

  // Constructors
  // Default

  Testing("OOMPI_Datatype::Constructors");
  {
    Testing("OOMPI_Datatype::Default constructor");
    OOMPI_Datatype t1;
    Test(t1.Is_null());
    
    Testing("OOMPI_Datatype::MPI constructor");
    OOMPI_Datatype t2(MPI_INT);
    Test(t2.Get_mpi() == MPI_INT && 
	 t2.Get_tag() == OOMPI_MPI_DATATYPE_TAG &&
	 t2.Built());

    Testing("OOMPI_Datatype::Copy constructor");
    OOMPI_Datatype t3(t2);
    Test(t3.Get_mpi() == MPI_INT && 
	 t3.Get_tag() == OOMPI_MPI_DATATYPE_TAG &&
	 t3.Built());

    Testing("OOMPI_Datatype::operator=()");
    OOMPI_Datatype t4;
    t4 = t3;
    t4.Get_mpi();
    t4.Get_tag();
    t4.Built();
    Test(t4.Get_mpi() == MPI_INT && 
	 t4.Get_tag() == OOMPI_MPI_DATATYPE_TAG &&
	 t4.Built());
  }
  Pass(); // Constructors

  Testing("OOMPI_Datatype::Is_null()");
  {
    OOMPI_Datatype a, b(OOMPI_INT);
    Test(a.Is_null() && !b.Is_null());
  }

  // Builtin types

  Testing("OOMPI_Datatype::Builtin types");
  {
    Testing("OOMPI_Datatype::Contiguous");
    Postpone("OOMPI_Port");
    
    Testing("OOMPI_Datatype::Vector");
    Postpone("OOMPI_Port");
    
    Testing("OOMPI_Datatype::Hvector");
    Postpone("OOMPI_Port");
    
    Testing("OOMPI_Datatype::Indexed");
    Postpone("OOMPI_Port");
    
    Testing("OOMPI_Datatype::Hindexed");
    Postpone("OOMPI_Port");
  }
  Postpone("OOMPI_Port"); // Builtin types

  // Type information
  // Extent

  Testing("OOMPI_Datatype::Type information");
  {
    Testing("OOMPI_Datatype::Extent()");
    OOMPI_Datatype t2(MPI_INT);

    MPI_Type_extent(MPI_INT, &ext);
    
    MPI_Type_contiguous(45, MPI_INT, &mpi_d1);
    OOMPI_Datatype oompi_d1(mpi_d1);
    MPI_Type_extent(mpi_d1, &ext1);
    
    MPI_Type_vector(45, 1, 50, MPI_INT, &mpi_d2);
    OOMPI_Datatype oompi_d2(mpi_d2);
    MPI_Type_extent(mpi_d2, &ext2);
    
    Test(t2.Extent() == ext &&
	 oompi_d1.Extent() == ext1 &&
	 oompi_d2.Extent() == ext2);
    
    // Size
    
    Testing("OOMPI_Datatype::Size()");
    MPI_Type_size(MPI_INT, &size);
    MPI_Type_size(mpi_d1, &size1);
    MPI_Type_size(mpi_d2, &size2);
    Test(t2.Size() == size &&
	 oompi_d1.Size() == size1 &&
	 oompi_d2.Size() == size2);
    
    // Lb
    
    Testing("OOMPI_Datatype::Lb()");
    MPI_Type_lb(MPI_INT, &low);
    MPI_Type_lb(mpi_d1, &low1);
    MPI_Type_lb(mpi_d2, &low2);
    Test(t2.Lb() == low &&
	 oompi_d1.Lb() == low1 &&
	 oompi_d2.Lb() == low2);
    
    // Ub
    
    Testing("OOMPI_Datatype::Ub()");
    MPI_Type_lb(MPI_INT, &hi);
    MPI_Type_lb(mpi_d1, &hi1);
    MPI_Type_lb(mpi_d2, &hi2);
    Test(t2.Ub() == hi &&
	 oompi_d1.Ub() == hi1 &&
	 oompi_d2.Ub() == hi2);

  }    

  MPI_Type_free(&mpi_d1);
  MPI_Type_free(&mpi_d2);

  Pass(); // Type information

  // Structure datatypes

  Testing("OOMPI_Datatype::Type struct");
  My_object *mine = new My_object;
  Pass(); // Type struct stuff

  // Destructor

  Testing("OOMPI_Datatype::Destructor");
  {
    OOMPI_Datatype &type = mine->Get_type();
    delete mine;
    Test(!type.Is_null() && type.Built());
  }

  Pass(); // OOMPI_Datatype
}
