// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: group_test.cc,v 1.6 2003/02/13 21:07:48 jsquyres Exp $
//
// OOMPI test program
// Test OOMPI_Group functionality
//

#include "oompi.h"
#include "oompi_test.h"


//
// OOMPI_Group_test
//
void
OOMPI_Group_test()
{
  int i;
  Testing("OOMPI_Group");

  Testing("OOMPI_Group::Default Constructor"); 
  OOMPI_Group oompi_group;
  Test(oompi_group.Is_null());

  Testing("OOMPI_Group::MPI Constructor"); 
  MPI_Group MPI_GROUP_WORLD;
  MPI_Comm_group(MPI_COMM_WORLD, &MPI_GROUP_WORLD);
  OOMPI_Group A(MPI_GROUP_WORLD);
  int result;
  MPI_Group_compare(A.Get_mpi(), MPI_GROUP_WORLD, &result);
  Test(result == MPI_IDENT);
  
  Testing("OOMPI_Group::Copy Constructor");
  OOMPI_Group B(A);
  MPI_Group_compare(B.Get_mpi(), MPI_GROUP_WORLD, &result);
  Test(result == MPI_IDENT);
  
  Testing("OOMPI_Group::operator=()");
  OOMPI_Group C;
  C = A;
  MPI_Group_compare(C.Get_mpi(), MPI_GROUP_WORLD, &result);
  Test(result == MPI_IDENT);

  Testing("OOMPI_Group::Destructor");
  for (i= 0; i < 1; i++) {
    OOMPI_Group D(A);
    i++; // Fiddle so that it is not optimized out
  }
  Pass();

  Testing("OOMPI_Group::Is_null()");
  {
    OOMPI_Group yuk;
    Test(yuk.Is_null() && !A.Is_null());
  }

  Testing("OOMPI_Group::Is_empty()");
  {
    OOMPI_Group a(MPI_GROUP_EMPTY);
    Test(a.Is_empty() && !A.Is_empty());
  }

  OOMPI_Group gw(A);

  Testing("OOMPI_Group::Size()");
  Test(OOMPI_COMM_WORLD.Size() == gw.Size());

  Testing("OOMPI_Group::Rank()");
  Test(OOMPI_COMM_WORLD.Rank() == gw.Rank());

  Testing("OOMPI_Group::Comparisons()");
  Testing("OOMPI_Group::Compare()", MPI_COMM_NULL);
  int mpi_compare1, mpi_compare2, mpi_compare3;
  OOMPI_Intra_comm comm1 = OOMPI_COMM_WORLD.Split(my_rank % 2);
  OOMPI_Intra_comm comm2 = OOMPI_COMM_WORLD.Split(0, -my_rank);
  
  OOMPI_Group g1 = comm1.Group();
  OOMPI_Group g2 = comm2.Group();
  
  MPI_Group_compare(MPI_GROUP_WORLD, g1.Get_mpi(), &mpi_compare1);
  MPI_Group_compare(g1.Get_mpi(), g1.Get_mpi(), &mpi_compare2);
  MPI_Group_compare(g2.Get_mpi(), gw.Get_mpi(), &mpi_compare3);
  
  Test(g1.Compare(gw) == mpi_compare1 &&
       g1.Compare(g1) == mpi_compare2 &&
       g2.Compare(gw) == mpi_compare3);
    
  Testing("OOMPI_Group::operator==()", MPI_COMM_NULL);
  Test(g1 == g1 && gw == g2);
  
  Testing("OOMPI_Group::operator!=()", MPI_COMM_NULL);
  Test(g1 != gw && g2 != g1);
  
  Pass(); // Comparisons

  Testing("OOMPI_Group::Translate_ranks()");
  {
    int *ranks1 = new int[comm_size];
    int *ranks2 = new int[comm_size];
    int *ranks3;

    for (i = 0; i < comm_size; i++)
      ranks1[i] = i;
    g2.Translate_ranks(comm_size, ranks1, gw, ranks2);
    for (i = 0; i < comm_size; i++)
      Midtest(ranks1[i] == ranks2[comm_size - 1 - i]);
    Sync();

    ranks3 = g2.Translate_ranks(comm_size, ranks1, gw);
    for (i = 0; i < comm_size; i++)
      Midtest(ranks1[i] == ranks2[comm_size - 1 - i]);

    delete[] ranks1;
    delete[] ranks2;
    delete[] ranks3;
  }
  Pass();
  
  Testing("OOMPI_Group::Incl()");
  OOMPI_Group OOMPI_GROUP_WORLD(A);

  int ranks1[] = {0}; 
  int n1 = 1;
  OOMPI_Group g3 = OOMPI_GROUP_WORLD.Incl(n1,ranks1);
  MPI_Group m1;
  int result1;
  MPI_Group_incl(MPI_GROUP_WORLD, n1, ranks1, &m1);
  MPI_Group_compare(g3.Get_mpi(), m1, &result1);
  MPI_Group_free(&m1);
  Test(result1 == MPI_IDENT);
  
  Testing("OOMPI_Group::Excl()");
  int n2 = OOMPI_COMM_WORLD.Size() - 1;
  int *ranks2 = new int [n2];
  for (i = 0; i < n2; i++) ranks2[i] = i+1;
  OOMPI_Group g4 = OOMPI_GROUP_WORLD.Excl(n2,ranks2);
  Test(g3.Compare(g4) == MPI_IDENT);
  delete[] ranks2;

  Testing("OOMPI_Group::Range_incl()");
  if (flags[SKIP_IBM21014])
    Done("Skipped (IBM 2.1.0.14)");
  else {
    int ranges1[1][3];
    ranges1[0][0] = 0;
    ranges1[0][1] = 0;
    ranges1[0][2] = 1;

    OOMPI_Group g5 = OOMPI_GROUP_WORLD.Range_incl(1, ranges1);
    Test(g5.Compare(g3) == MPI_IDENT);
  }
  
  int ranges2[1][3];
  ranges2[0][0] = 1;
  ranges2[0][1] = n2;
  ranges2[0][2] = 1;
  
  Testing("OOMPI_Group::Range_excl()");
  if (flags[SKIP_IBM21014])
    Done("Skipped (IBM 2.1.0.14)");
  else {
    OOMPI_Group g6 = OOMPI_GROUP_WORLD.Range_excl(1,ranges2);
    Test(g6.Compare(g3) == OOMPI_IDENT);
  }

 
  {
    int *granks1 = new int [ comm_size];
    int *granks2 = new int [ comm_size];
    for (i=0; i< comm_size/2; i++) {
      granks1[i] = i*2;
      granks2[i] = i*2+1;
    }
    OOMPI_Group gr1 = OOMPI_GROUP_WORLD.Excl(comm_size/2, granks1);
    OOMPI_Group gr2 = OOMPI_GROUP_WORLD.Excl(comm_size/2, granks2);
    
    Testing("OOMPI_Group::operator|()");
    OOMPI_Group gr3 = gr1 | gr2; 
    Test(gr3 == OOMPI_GROUP_WORLD);
    
    Testing("OOMPI_Group::operator&()");
    OOMPI_Group gr5 = gr1 & gr2;
    OOMPI_Group gr6 = gr1 & OOMPI_GROUP_WORLD;
    Test(gr5.Is_empty() &&
	 gr6.Compare(gr1) == OOMPI_IDENT);
    
    Testing("OOMPI_Group::operator-()");
    Test(OOMPI_GROUP_WORLD - gr1 == gr2 &&
	 OOMPI_GROUP_WORLD - gr2 == gr1);

    delete [] granks1;
    delete [] granks2;
  }
  Pass();
}


