// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: intra_comm_test.cc,v 1.11 2003/02/13 21:07:48 jsquyres Exp $
//
// OOMPI test program
// Test OOMPI_Intra_comm functionality
//

#include <iostream>

#include "oompi.h"
#include "oompi_test.h"


//
// Convenience function
//
static void inline
Testarray(double* a, double* b, int size, const char* msg)
{
  int i;

  for (i = 0; i < size; i++) {
    if (a[i] != b[i]) {
      std::cout << my_rank << ": a[" << i << "]: " << a[i] << " b[" 
		<< i << "]: " << b[i] << std::endl;
      Fail(msg);
    }
  }
  Sync();
};


//
// OOMPI_Intra_comm_test
//
void
OOMPI_Intra_comm_test()
{
  int result, i, l;
  Testing("OOMPI_Intra_comm");

  // Copy constructor

  Testing("OOMPI_Intra_comm::Copy constructor");
  OOMPI_Intra_comm comm1(OOMPI_COMM_WORLD);
  MPI_Comm mpi_comm1 = MPI_COMM_WORLD;
  MPI_Comm_compare(OOMPI_COMM_WORLD.Get_mpi(), mpi_comm1, &result);
  Test(result == MPI_IDENT);

  // operator=

  Testing("OOMPI_Intra_comm::operator=()");
  comm1 = OOMPI_COMM_WORLD;
  MPI_Send(&result, 1, MPI_INT, MPI_PROC_NULL, 1001, comm1.Get_mpi());
  MPI_Send(&result, 1, MPI_INT, MPI_PROC_NULL, 1001, mpi_comm1);
  MPI_Comm_compare(comm1.Get_mpi(), mpi_comm1, &result);
  //MPI_Comm_compare(MPI_COMM_WORLD, MPI_COMM_WORLD, &result);
  Test(result == MPI_IDENT);

  // Dup

  Testing("OOMPI_Intra_comm::Dup()");
  MPI_Comm mpi_comm2;
  MPI_Comm_dup(MPI_COMM_WORLD, &mpi_comm2);
  OOMPI_Intra_comm comm2 = comm1.Dup();

  MPI_Comm_compare(comm2.Get_mpi(), mpi_comm2, &result);
  Test(result == MPI_CONGRUENT);

  // Split

  Testing("OOMPI_Intra_comm::Split()");
  OOMPI_Intra_comm comm3 = comm1.Split(0, -my_rank);
  MPI_Comm mpi_comm3;
  MPI_Comm_split(mpi_comm1, 0, -my_rank, &mpi_comm3);

  MPI_Comm_compare(mpi_comm1, mpi_comm3, &result);
  if (!(result == MPI_SIMILAR))
    Fail("MPI_SIMILAR != MPI_UNEQUAL");

  OOMPI_Intra_comm comm4 = comm1.Split(my_rank, 0);
  MPI_Comm mpi_comm4;
  MPI_Comm_split(mpi_comm1, my_rank, 0, &mpi_comm4);

  MPI_Comm_compare(mpi_comm1, mpi_comm4, &result);
  if (!(result == MPI_UNEQUAL))
    Fail("MPI_UNEQUAL != MPI_UNEQUAL");
  Pass();

  MPI_Comm_free(&mpi_comm2);
  MPI_Comm_free(&mpi_comm3);
  MPI_Comm_free(&mpi_comm4);

  // OOMPI_Comm base class
  // Is_null

  Testing("OOMPI_Intra_comm::OOMPI_Comm base class");
  Testing("OOMPI_Comm::Is_null");
  {
    OOMPI_Intra_comm a;
    Midtest(!OOMPI_COMM_WORLD.Is_null() && a.Is_null());
  }
  Pass();

  // Compare

  Testing("OOMPI_Comm::Compare()");
  if (!(OOMPI_COMM_WORLD.Compare(comm1) == OOMPI_IDENT))
    Fail();
  if (!(comm1.Compare(OOMPI_COMM_WORLD) == OOMPI_IDENT))
    Fail();
  if (!(comm1.Compare(comm2) == OOMPI_CONGRUENT))
    Fail();
  if (!(comm2.Compare(comm1) == OOMPI_CONGRUENT))
    Fail();
  if (!(comm1.Compare(comm3) == OOMPI_SIMILAR))
    Fail();
  if (!(comm3.Compare(comm1) == OOMPI_SIMILAR))
    Fail();
  if (!(comm1.Compare(comm4) == OOMPI_UNEQUAL))
    Fail();
  if (!(comm4.Compare(comm1) == OOMPI_UNEQUAL))
    Fail();
  Pass();
  
  // operator ==
  
  Testing("OOMPI_Comm::operator==()");
  if (!(OOMPI_COMM_WORLD == comm1))
    Fail("MPI_IDENT != MPI_UNEQUAL");
  if (!(comm1 == OOMPI_COMM_WORLD))
    Fail("MPI_IDENT != MPI_UNEQUAL");
  if (!(comm1 == comm2))
    Fail("MPI_CONGRUENT != MPI_UNEQUAL");
  if (!(comm2 == comm1))
    Fail("MPI_CONGRUENT != MPI_UNEQUAL");
  if (!(comm1 == comm3))
    Fail("MPI_SIMILAR != MPI_UNEQUAL");
  if (!(comm3 == comm1))
    Fail("MPI_SIMILAR != MPI_UNEQUAL");
  if (comm1 == comm4)
    Fail("MPI_UNEQUAL != MPI_UNEQUAL");
  if (comm4 == comm1)
    Fail("MPI_UNEQUAL != MPI_UNEQUAL");
  Pass();
  
  // operator !=
  
  Testing("OOMPI_Comm::operator!=()");
  if (OOMPI_COMM_WORLD != comm1)
    Fail("MPI_IDENT != MPI_UNEQUAL");
  if (comm1 != OOMPI_COMM_WORLD)
    Fail("MPI_IDENT != MPI_UNEQUAL");
  if (comm1 != comm2)
    Fail("MPI_CONGRUENT != MPI_UNEQUAL");
  if (comm2 != comm1)
    Fail("MPI_CONGRUENT != MPI_UNEQUAL");
  if (comm1 != comm3)
    Fail("MPI_SIMILAR != MPI_UNEQUAL");
  if (comm3 != comm1)
    Fail("MPI_SIMILAR != MPI_UNEQUAL");
  if (!(comm1 != comm4))
    Fail("MPI_UNEQUAL != MPI_UNEQUAL");
  if (!(comm4 != comm1))
    Fail("MPI_UNEQUAL != MPI_UNEQUAL");
  Pass();
  
  // Group
  
  Testing("OOMPI_Comm::Group()");
  OOMPI_Group group1 = comm1.Group();
  MPI_Group mpi_group1;
  MPI_Comm_group(mpi_comm1, &mpi_group1);
  MPI_Group_compare(group1.Get_mpi(), mpi_group1, &result);
  if (result != MPI_IDENT)
    Fail("result != MPI_IDENT");
  
  OOMPI_Group group3 = comm3.Group();
  MPI_Group_compare(group3.Get_mpi(), mpi_group1, &result);
  if (result != MPI_SIMILAR)
    Fail("result != MPI_SIMILAR");
  
  OOMPI_Group group4 = comm4.Group();
  MPI_Group_compare(group4.Get_mpi(), mpi_group1, &result);
  if (result != MPI_UNEQUAL)
    Fail("result != MPI_UNEQUAL");
  Pass();
  
  Pass(); // OOMPI_Comm_world
  
  // Create

  Testing("OOMPI_Intra_comm::Create()");
  OOMPI_Intra_comm comm5 = comm4.Create(group4);
  if (comm1 == comm5)
    Fail("result should not be !(MPI_UNEQUAL)");
  if (comm5 != comm4)
    Fail("result should not be MPI_UNEQUAL");
  if (comm5.Compare(comm4) != OOMPI_CONGRUENT || 
      comm5.Compare(comm5) != OOMPI_IDENT ||
      comm5.Compare(comm3) != OOMPI_UNEQUAL)
    Fail("group comparisons failed");
  Pass();

  // Intercomm_create

  Testing("OOMPI_Intra_comm::OOMPI_Intercomm_create()");
  Postpone("OOMPI_Inter_comm");

  // Test_inter

  Testing("OOMPI_Intra_comm::Test_inter()");
  Test(!comm1.Test_inter());

  // Topo_test

  Testing("OOMPI_Intra_comm::Topo_test()");
  Postpone("OOMPI_Graph_comm");

  // Initialized

  Testing("OOMPI_Intra_comm::Initialized()");
  Test(comm1.Initialized());

  // Collectives
  // Barrier

  Testing("OOMPI_Intra_comm::Collective calls");
  {
    Testing("OOMPI_Intra_comm::Barrier()");
    OOMPI_COMM_WORLD.Barrier();
    Pass(); 
    
    // Allgather
    
    Testing("OOMPI_Intra_comm::Allgather()");
    {
      const int size = 5;
      int *send = new int[size];
      int *recv = new int[comm_size * size];
      int *recvcounts = new int[comm_size * size];
      int *displs = new int[comm_size * size];
      
      for (i = 0; i < size; i++)
	send[i] = my_rank;
      OOMPI_Message send_msg(send, size);
      OOMPI_Message recv_msg(recv, size);
      
      // Test 1

      for (i = 0; i < comm_size * size; i++)
	recv[i] = -1;
      comm1.Allgather(send_msg, recv_msg);
      for (i = 0; i < comm_size; i++)
	for (l = 0; l < size; l++)
	  if (recv[i * size + l] != i)
	    Fail("Message / Message");
      Sync();
      
      // Test 2
      
      for (i = 0; i < comm_size * size; i++)
	recv[i] = -1;
      comm1.Allgather(send_msg, recv, size);
      for (i = 0; i < comm_size; i++)
	for (l = 0; l < size; l++)
	  if (recv[i * size + l] != i)
	    Fail("Message / Array_message");
      Sync();
      
      // Test 3
      
      for (i = 0; i < comm_size * size; i++)
	recv[i] = -1;
      comm1.Allgather(send, size, recv_msg);
      for (i = 0; i < comm_size; i++)
	for (l = 0; l < size; l++)
	  if (recv[i * size + l] != i)
	    Fail("Array_message / Message");
      Sync();
      
      // Test 4
      
      for (i = 0; i < comm_size * size; i++)
	recv[i] = -1;
      comm1.Allgather(send, size, recv, size);
      for (i = 0; i < comm_size; i++)
	for (l = 0; l < size; l++)
	  if (recv[i * size + l] != i)
	    Fail("Array_message / Array_message");
      Pass();
      
      // Allgatherv
      // If you can figure out what this does...

      Testing("OOMPI_Intra_comm::Allgatherv()");
      {
	send[0] = my_rank;
	for (i = 0; i < comm_size; i++) {
	  recvcounts[i] = 1;
	  displs[i] = i * 2;
	}
	comm1.Allgatherv(*send, recv, recvcounts, displs);
	for (i = 0; i < comm_size; i++)
	  if (recv[i * 2] != i)
	    Fail("Message / Array_message");
	Sync();
      }

      {
	int pos, total = 0 ,sum= 0;
	for (i = 0; i < comm_size; i++)
	  total += i + 1;
	delete[] recv;
	recv = new int[total + 1];
	sum = total;

	delete[] send;
	send = new int[comm_size + 1];
	for (i = 0; i <= my_rank; i++)
	  send[i] = my_rank;
	for (sum--, i = 0; i < comm_size; i++) {
	  recvcounts[i] = i + 1;
	  displs[i] = sum;
	  sum -= i + 2;
	}
	comm1.Allgatherv(send, my_rank + 1, recv, recvcounts, displs);
	for (pos = total - 1, i = 0; i < comm_size; i++)
	  for (l = 0; l <= i; l++, pos--)
	    if (recv[pos] != i)
	      Fail("Array_message / Array_message");
      }
	
      delete[] send;
      delete[] recv;
      delete[] recvcounts;
      delete[] displs;
    }
    Pass(); // Actually, Allgatherv, but they share data
    
    // Allreduce

    Testing("OOMPI_Intra_comm::Allreduce()"); 
    {
      const int size = 5;
      int *send = new int[size * 2];
      int *recv = new int[comm_size * size * 2];
      int total = 0, sum = 0;
      for (i = 0; i < comm_size; i++)
	total += i;

      comm1.Allreduce(my_rank, sum, OOMPI_SUM);
      Midtest(sum == total, "Message / Array_message");
      
      send[0] = send[1] = my_rank;
      comm1.Allreduce(send, 2, recv, OOMPI_SUM);
      if (recv[0] != total || recv[1] != total)
	Fail("Array_message / Array_message");
      delete[] send;
      delete[] recv;
    }
    Pass();
    
    // Alltoall

    Testing("OOMPI_Intra_comm::Alltoall()");
    {
      double *dsend = new double[comm_size];
      double *drecv = new double[comm_size];
      double result2;
      for (i = 0; i < comm_size; i++)
	dsend[i] = (i + 1) * 123.456;

      // Test 1

      for (i = 0; i < comm_size; i++)
	drecv[i] = 0;
      OOMPI_Message send_msg(dsend, 1);
      OOMPI_Message recv_msg(drecv, 1);
      comm1.Alltoall(send_msg, recv_msg);
      result2 = (my_rank + 1) * 123.456;
      for (i = 0; i < comm_size; i++)
	Midtest(doublecmp(drecv[i], result2), "Message / Message");

      // Test 2

      for (i = 0; i < comm_size; i++)
	drecv[i] = 0.0;
      comm1.Alltoall(dsend[0], drecv, 1);
      for (i = 0; i < comm_size; i++)
	Midtest(doublecmp(drecv[i], result2), "Message / Array_message");

      // Test 3

      for (i = 0; i < comm_size; i++)
	drecv[i] = 0.0;
      comm1.Alltoall(dsend, 1, recv_msg);
      for (i = 0; i < comm_size; i++)
	Midtest(doublecmp(drecv[i], result2), "Array_message / Message");

      // Test 4

      for (i = 0; i < comm_size; i++)
	drecv[i] = 0.0;
      comm1.Alltoall(dsend, 1, drecv, 1);
      for (i = 0; i < comm_size; i++)
	Midtest(doublecmp(drecv[i], result2), "Array_message / Array_message");

      delete[] dsend;
      delete[] drecv;
    }
    Pass();
    
    // Alltoallv
    
    Testing("OOMPI_Intra_comm::Alltoallv()");
    {
      // Make a bizarre pattern:
      // send_buf will have a pattern like:
      //   x o x x o x x x o x x x x o ....
      //   where x is data, and o is a hole
      //   each group will have data of X, 2X, 3X, etc...

      int *scounts = new int[comm_size];
      int *counts = new int[comm_size];
      int *sdispls = new int[comm_size];
      int *displs = new int[comm_size];
      for (i = 0; i < comm_size; i++) {
	scounts[i] = counts[i] = comm_size;
	sdispls[i] = displs[i] = i * (comm_size + 1);
      }

      int size = (comm_size + 1) * comm_size;
      double *dsend = new double[size];
      double *drecv = new double[size];
      double result2;

      for (i = 0; i < comm_size; i++)
	for (l = 0; l < comm_size; l++)
	  dsend[(i * (comm_size + 1)) + l] = (i + 1) * 234.567;

      // Test 1

      for (i = 0; i < size; i++)
	drecv[i] = 0.0;
      comm1.Alltoallv(dsend, counts, displs, drecv, counts, displs);

      result2 = ((my_rank + 1) * 234.567);
      for (i = 0; i < comm_size; i++)
	for (l = 0; l < comm_size; l++)
	  Midtest(doublecmp(drecv[(i * (comm_size + 1)) + l], result2), 
		  "Alltoallv");

      delete[] dsend;
      delete[] drecv;
      delete[] scounts;
      delete[] counts;
      delete[] sdispls;
      delete[] displs;
    }
    Pass();
    
    // Reduce_scatter
    
    Testing("OOMPI_Intra_comm::Reduce_scatter()");
    {
      int sum = 0, pos;
      for (i = 0; i < comm_size; i++)
	sum += (i + 1);
      double *send = new double[sum];
      double *recv = new double[my_rank + 1];
      double result1, result2;

      // Test 1

      int *counts = new int[comm_size];
      counts[0] = 1;
      for (i = 1; i < comm_size; i++) 
	counts[i] = 0;
      send[0] = 234.567;
      recv[0] = 0.0;
      comm1.Reduce_scatter(send[0], recv, counts, OOMPI_SUM);
      result1 = 234.567 * comm_size;
      result2 = 0.0;
      Midtest((my_rank == 0 && doublecmp(recv[0], result1)) ||
	      (my_rank != 0 && doublecmp(recv[0], result2)));

      // Test 2

      for (pos = i = 0; i < comm_size; i++) {
	counts[i] = i + 1;
	for (l = 0; l <= i; l++, pos++)
	  send[pos] = (i + 1) * 234.567;
      }

      for (i = 0; i <= my_rank; i++)
	recv[i] = 0.0;
      comm1.Reduce_scatter(send, recv, counts, OOMPI_SUM);
      result1 = (my_rank + 1) * 234.567 * comm_size;
      for (i = 0; i < comm_size; i++)
	if (i <= my_rank) 
	  Midtest(doublecmp(recv[i], result1), 
		  "Array_message / Array_message");
	else
	  Sync();
      
      delete[] counts;
      delete[] send;
      delete[] recv;
    }
    Pass();
    
    // Scan
    
    Testing("OOMPI_Intra_comm::Scan()");
    {
      double *dsend = new double [comm_size];
      double *drecv = new double [comm_size];
      double *sum = new double [comm_size];
      for (i = 0; i < comm_size; i++) {
	dsend[i] = i;
	drecv[i] = 0.0;
      }
      
      // Test 1
      
      {
	OOMPI_Message send_msg(dsend, comm_size);
	comm1.Scan(send_msg, drecv, OOMPI_SUM);
	for (i = 0; i < comm_size; i++)
	  sum[i] = dsend[i] * (my_rank + 1);
	Testarray(sum, drecv, comm_size, "Message / Array_message");
      }
      
      // Test 2
      
      {
	for (i = 0; i < comm_size; i++)
	  drecv[i] = 0.0;
	comm1.Scan(dsend, drecv, comm_size, OOMPI_SUM);
	for (i = 0; i < comm_size; i++)
	  sum[i] = dsend[i] * (my_rank + 1);
	Testarray(sum, drecv, comm_size, "Message / Array_message");
      }
      
      delete[] dsend;
      delete[] drecv;
      delete[] sum;
      
      Pass();
    }
  }
  Pass(); // Collective calls

  // ANY_SOURCE calls
  // operator>>

  Testing("OOMPI_Intra_comm::ANY_SOURCE calls");
  {
    // Ring-like structure doesn't work here
    // Use odd/even rank pairs exchange messages

    to = from = ((my_rank % 2) == 0) ? (my_rank + 1) : (my_rank - 1);

    Testing("OOMPI_Intra_comm::operator>>()");
    {
      l = -1;
      if ((my_rank % 2) == 0) {
	OOMPI_COMM_WORLD[to].Send(my_rank);
	Sync("ANY_SOURCE to odds");
	OOMPI_COMM_WORLD >> l;
	Midtest(l == from, "ANY_SOURCE to evens");
      }
      else {
	OOMPI_COMM_WORLD >> l;
	Midtest(l == from, "ANY_SOURCE to odds");
	OOMPI_COMM_WORLD[to].Send(my_rank);
	Sync("ANY_SOURCE to evens");
      }
    }
    Pass();
      
    Testing("OOMPI_Intra_comm::Recv()");
    {
      l = -1;
      if ((my_rank % 2) == 0) {
	OOMPI_COMM_WORLD[to].Send(my_rank);
	Sync("ANY_SOURCE to odds");
	OOMPI_COMM_WORLD.Recv(l);
	Midtest(l == from, "ANY_SOURCE to evens");
      }
      else {
	OOMPI_COMM_WORLD.Recv(l);
	Midtest(l == from, "ANY_SOURCE to odds");
	OOMPI_COMM_WORLD[to].Send(my_rank);
	Sync("ANY_SOURCE to evens");
      }
    }
    Pass();
    
    Testing("OOMPI_Intra_comm::Irecv()");
    {
      OOMPI_Request_array req(2);
      
      l = -1;
      req[0] = OOMPI_COMM_WORLD[to].Isend(my_rank);
      req[1] = OOMPI_COMM_WORLD.Irecv(l);
      req.Waitall();
      Midtest(l == from);
    }
    Pass();
    
    Testing("OOMPI_Intra_comm::Recv_init()");
    {
      OOMPI_Request_array req(2);
      
      req[1] = OOMPI_COMM_WORLD.Recv_init(l);
      for (i = 0; i < 5; i++) {
	req[0] = OOMPI_COMM_WORLD[to].Isend(my_rank);
	l = -1;
	req[1].Start();
	req.Waitall();
	Midtest(l == from);
      }
      req[1].Free();
    }
    Pass();
    
    Testing("OOMPI_Intra_comm::Probe()");
    Postpone("OOMPI_Status");
    
    Testing("OOMPI_Intra_comm::Iprobe()");
    Postpone("OOMPI_Status");
  }
  Pass();
  
  // Topology info
  
  Testing("OOMPI_Intra_comm::Topology info");
  {
    Testing("OOMPI_Intra_comm::Graph_map()");
    {
      int nnodes = comm_size;
      int nedges = comm_size * 2;
      int *index = new int [nnodes];
      int *edges = new int [nedges];
 
      for (i=0; i<nnodes; i++) {
        index[i] = (2*i+2);
        edges[2*i] = (i+1) % comm_size;
	edges[2*i+1] = (i-1+comm_size) % comm_size;
      }
 
      int oompi_rank = OOMPI_COMM_WORLD.Graph_map(nnodes, index, edges);
      int mpi_rank;
      MPI_Graph_map(MPI_COMM_WORLD, nnodes, index, edges, &mpi_rank);
      Midtest(mpi_rank == oompi_rank);
      delete[] index;
      delete[] edges;
    }
    Pass();

    Testing("OOMPI_Intra_comm::Cart_map()");
    {
      int ndims = 4;
      int dims[4] = {1,0,0,1};
      bool periods_bool[4] = { false, false, false, false };
      int periods_int[4] = { 0, 0, 0, 0 };
 
      MPI_Dims_create(comm_size, ndims, dims);
 
      int oompi_rank = OOMPI_COMM_WORLD.Cart_map(ndims, dims, periods_bool);
      int mpi_rank;
      MPI_Cart_map(MPI_COMM_WORLD, ndims, dims, periods_int, &mpi_rank);
      Midtest(mpi_rank == oompi_rank);
      oompi_rank =  OOMPI_COMM_WORLD.Cart_map(ndims, dims, false);
      Midtest(mpi_rank == oompi_rank);
    }
    Pass();

    Testing("OOMPI_Intra_comm::Dims_create()");
    {
      int dims[3] = {0,0,0};
      int *out;

      // Only one dimension.  
      out = OOMPI_COMM_WORLD.Dims_create(1, dims);
      Midtest(out == dims && dims[0] == comm_size,
	      "One Dimension Test");

      // Two dimensions, but one of them is specified
      if (comm_size % 2 == 0) {
	dims[0] = 2;
	dims[1] = 0;
	out = OOMPI_COMM_WORLD.Dims_create(2, dims);
	Midtest(out == dims && dims[0] == 2 && dims[1] == comm_size / 2,
		"Two Dimension Test 1");
	
	dims[0] = 0;
	dims[1] = 2;
	out = OOMPI_COMM_WORLD.Dims_create(2, dims);
	Midtest(out == dims && dims[0] == comm_size / 2  && dims[1] == 2,
		"Two Dimension Test 2");
      }
      
      // Test in 3 dimensions - simple test
      dims[0] = 0;
      dims[1] = 0;
      dims[2] = 0;
      
      // Assuming we have at least 2 processors.  This test 
      // another entry point into the function.
      out = OOMPI_COMM_WORLD.Dims_create(3, dims, 2);
      Midtest(out == dims && dims[0]*dims[1]*dims[2] == 2, 
	      "Three Dimension Test");
    }
    Pass();
    
  }
  Pass(); // Topology info
  
  Pass(); // OOMPI_Intra_comm
}

