// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: oompi_test.h,v 1.7 2003/02/13 21:07:48 jsquyres Exp $
//
// OOMPI test suite
// Test *all* OOMPI functionality
//

#ifndef _OOMPI_TEST_H_
#define _OOMPI_TEST_H_

#include <math.h>
#include "mpi.h"


// Version number

extern const int version[3];


// All the testing functions

void OOMPI_Environment_test();

void OOMPI_Comm_world_test();
void OOMPI_Comm_test();
void OOMPI_Intra_comm_test();
void OOMPI_Inter_comm_test();
void OOMPI_Cart_comm_test();
void OOMPI_Graph_comm_test();

void OOMPI_Group_test();

void OOMPI_Op_test();
void OOMPI_Port_test();

void OOMPI_Message_test();
void OOMPI_Array_message_test();
void OOMPI_Tag_test();

void OOMPI_Datatype_test();
void OOMPI_User_type_test();
void OOMPI_Packed_test();

void OOMPI_Request_test();
void OOMPI_Request_array_test();
void OOMPI_Status_test();
void OOMPI_Status_array_test();


// Helper functions
// oompi_test.cc

extern int my_rank;
extern int comm_size;
extern int to;
extern int from;
extern const double Epsilon;

void do_work(int top = -1);


// messages.cc

void Testing(const char* msg, MPI_Comm comm = MPI_COMM_WORLD);
void Pass(const char* msg = "PASS");
void Sync(const char* msg = 0);
void Postpone(const char* class_name);
void Done(const char* msg = 0);
void Fail(const char* msg = "FAIL");
void Abort(const char* msg = 0);


// stack.cc

void Push(const char* msg);
char* Pop();


// signal.cc

void signal_init();


// General helper functions

inline void Test(int c, const char* msg = 0) 
{ (c) ? Pass() : Fail(msg); };
inline void Midtest(int c, const char*msg = 0) 
{ (c) ? Sync(msg) : Fail(msg); };
inline bool doublecmp(double a, double b) 
{ return (bool) (fabs(a - b) < Epsilon); };


// Skip test flags

typedef enum _skip_flags {
  SKIP_CRAY1104,
  SKIP_HPUX0102,
  SKIP_IBM21014, SKIP_IBM21015, SKIP_IBM21016, SKIP_IBM21017, SKIP_IBM21018,
  SKIP_IBM23012,
  SKIP_SGI20, SKIP_SGI30, SKIP_SGI32,
  SKIP_MAX
} SKIP_FLAGS;
extern bool flags[SKIP_MAX];


#endif // _OOMPI_TEST_H_
