// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: packed_test.cc,v 1.6 2003/02/13 21:07:48 jsquyres Exp $
//
// OOMPI test program
// Test OOMPI_Packed functionality
//

#include "oompi.h"
#include "oompi_test.h"


//
// OOMPI_Packed_test
//
void
OOMPI_Packed_test()
{
  int size;
  int idata[400];
  int i, j, k;
  int pos;
  double ddata[400];
  double l, m, n;
  Testing("OOMPI_Packed");

  // Allocating constructor

  Testing("OOMPI_Packed::Allocating constructor");
  size = OOMPI_COMM_WORLD.Pack_size(OOMPI_INT, 100);
  OOMPI_Packed alloc_buf(size, OOMPI_COMM_WORLD, 1234);
  Test(alloc_buf.Get_size() == size &&
       alloc_buf.Get_position() == 0 &&
       alloc_buf.Get_tag() == 1234);

  // Non-allocating constructor

  Testing("OOMPI_Packed::Non-allocating constructor");
  OOMPI_Packed nonalloc_buf(idata, 400 * sizeof(int), OOMPI_COMM_WORLD, 2345);
  Test(nonalloc_buf.Get_size() == (400 * sizeof(int)) &&
       nonalloc_buf.Get_position() == 0 &&
       nonalloc_buf.Get_tag() == 2345);

  // Get size

  Testing("OOMPI_Packed::Get_size()");
  Pass(); // Already passed, above

  // Reset

  Testing("OOMPI_Packed::Reset()");
  alloc_buf.Reset();
  Test(alloc_buf.Get_position() == 0);

  // Clear buffer (for copy constructor and destructor tests below)
  // This assumes that pack will work, even though it has not been
  // tested yet.

  alloc_buf.Reset();
  for(i = 0; i < 100; i++)
    alloc_buf.Pack(i);
  alloc_buf.Reset();

  // Pack

  Testing("OOMPI_Packed::Pack()");
  i = 1;
  j = 2;
  k = 3;
  l = 4.5;
  m = 7.0/8.0;
  n = 22.0/7.0;

  idata[0] = 938;
  idata[1] = 389;
  idata[2] = 893;
  ddata[0] = 123.456;
  ddata[1] = 789.012;
  ddata[2] = 345.678;

  alloc_buf.Pack(i);
  alloc_buf.Pack(l);
  alloc_buf.Pack(j);
  alloc_buf.Pack(idata, 3);
  alloc_buf.Pack(m);
  alloc_buf.Pack(k);
  alloc_buf.Pack(n);
  alloc_buf.Pack(ddata, 3);
  pos = alloc_buf.Get_position();
  Test(pos != 0);

  // Set size

  Testing("OOMPI_Packed::Set_size()");
  alloc_buf.Set_size(size * 2);
  Test(alloc_buf.Get_size() == size * 2);
  alloc_buf.Set_size(size);

  // Streams pack

  Testing("OOMPI_Packed::operator<<()");
  alloc_buf << i << l << j << m << k << n;
  Test(alloc_buf.Get_position() != pos);
  pos = alloc_buf.Get_position();

  // Unpack

  Testing("OOMPI_Packed::Unpack()");
  alloc_buf.Reset();
  alloc_buf.Unpack(j);
  alloc_buf.Unpack(m);
  alloc_buf.Unpack(k);
  alloc_buf.Unpack(&(idata[4]), 3);
  alloc_buf.Unpack(n);
  alloc_buf.Unpack(i);
  alloc_buf.Unpack(l);
  alloc_buf.Unpack(&(ddata[4]), 3);
  pos = alloc_buf.Get_position();
  Test(j == 1 && k == 2 && i == 3 &&
       m == 4.5 && n == (7.0/8.0) && l == (22.0/7.0) &&
       idata[4] == idata[0] && idata[5] == idata[1] && idata[6] == idata[2] &&
       ddata[4] == ddata[0] && ddata[5] == ddata[1] && ddata[6] == ddata[2] &&
       pos != 0);

  // Streams pack

  Testing("OOMPI_Packed::operator>>()");
  alloc_buf >> i >> l >> j >> m >> k >> n;
  Test(i == 1 && j == 2 && k == 3 &&
       l == 4.5 && m == (7.0/8.0) && n == (22.0/7.0) &&
       alloc_buf.Get_position() != pos);

  // Get_position

  Testing("OOMPI_Packed::Get_position()");
  {
    alloc_buf.Reset();
    alloc_buf << i;
    pos = alloc_buf.Get_position();
    Test(pos != 0);
  }

  // Set_position

  Testing("OOMPI_Packed::Set_position()");
  {
    double dummy;

    alloc_buf.Reset();
    alloc_buf << i;
    pos = alloc_buf.Get_position();
    alloc_buf << l;
    alloc_buf.Set_position(pos);
    alloc_buf >> dummy;
    Test(l == dummy);
  }

  // Start

  Testing("OOMPI_Packed::Start()");
  {
    double dummy;

    alloc_buf.Reset();
    alloc_buf << i;
    pos = alloc_buf.Get_position();
    alloc_buf << l;
    alloc_buf.Start(pos);
    alloc_buf >> dummy;
    Test(l == dummy);
  }

  // End

  Testing("OOMPI_Packed::End()");
  {
    int dummy;

    alloc_buf.Reset();
    alloc_buf << i;
    alloc_buf.End();
    alloc_buf >> dummy;
    Test(i == dummy);
  }

  // Copy constructor

  Testing("OOMPI_Packed::Copy constructor");
  alloc_buf.Reset();
#if 1
  alloc_buf << idata[0] << idata[1] << idata[2] << i << l << j << 
    ddata[0] << ddata[1] << ddata[2] << m << k << n;
#else
  OOMPI_Message i0(idata[0]), i1(idata[1]), i2(idata[2]), ii(i), il(l), ij(j), 
    d0(ddata[0]), d1(ddata[1]), d2(ddata[2]), dm(m), ik(k), dn(n);

  alloc_buf << i0 << i1 << i2 << ii << il << ij << d0 << d1 << d2 << 
    dm << ik << dn;
#endif
#if 1
  alloc_buf.Reset();
  alloc_buf >> idata[7] >> idata[8] >> idata[9] >> i >> l >> j >>
    ddata[7] >> ddata[8] >> ddata[9] >> m >> k >> n;
  OOMPI_Packed alloc2(alloc_buf);
  alloc2.Reset();
  alloc2 >> idata[7] >> idata[8] >> idata[9] >> i >> l >> j >> 
    ddata[7] >> ddata[8] >> ddata[9] >> m >> k >> n;
  alloc2.Reset();
  i = 11;
  j = 22;
  k = 33;
  alloc2 << i << j << k;
  alloc_buf.Reset();
  alloc_buf >> i >> j >> k;
  Test(alloc_buf.Get_size() == alloc2.Get_size() &&
       alloc_buf.Get_tag() == alloc2.Get_tag() && 
       idata[7] == idata[0] && idata[8] == idata[1] && idata[9] == idata[2] &&
       ddata[7] == ddata[0] && ddata[8] == ddata[1] && ddata[9] == ddata[2] &&
       i != 11 && j != 22 && k != 33);

  // Destructor

  alloc_buf.Reset();

  Testing("OOMPI_Packed::Destructor");
  for (i = 0; i < 1; i++) {
    OOMPI_Packed alloc3(alloc_buf);
    i++; // Fiddle so that optimizers don't get it
  }
  Test(alloc_buf.Get_size() == size &&
       alloc_buf.Get_position() == 0 &&
       alloc_buf.Get_tag() == 1234);

  // Assignment operator

  Testing("OOMPI_Packed::operator=()");
  nonalloc_buf= alloc_buf;
  nonalloc_buf.Reset();
  nonalloc_buf >> idata[7] >> idata[8] >> idata[9] >> i >> l >> j >> 
    ddata[7] >> ddata[8] >> ddata[9] >> m >> k >> n;
  nonalloc_buf.Reset();
  i= 11;
  j= 22;
  k= 33;
  nonalloc_buf << i << j << k;
  alloc_buf.Reset();
  alloc_buf >> i >> j >> k;
  Test(alloc_buf.Get_size() == nonalloc_buf.Get_size() &&
       alloc_buf.Get_tag() == nonalloc_buf.Get_tag() && 
       idata[7] == idata[0] && idata[8] == idata[1] && idata[9] == idata[2] &&
       ddata[7] == ddata[0] && ddata[8] == ddata[1] && ddata[9] == ddata[2] &&
       i != 11 && j != 22 && k != 33);
#endif
       
  Pass(); // OOMPI_Packed
}
