// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: request_array_test.cc,v 1.7 2003/02/13 21:07:48 jsquyres Exp $
//
// OOMPI test program
// Test OOMPI_Request_array functionality
//

#include "oompi.h"
#include "oompi_test.h"

void OOMPI_Request_array_test_wait();
void OOMPI_Request_array_test_test();
void OOMPI_Request_array_test_start();

//
// OOMPI_Request_array_test
//
void
OOMPI_Request_array_test()
{
  Testing("OOMPI_Request_array");

  int i;

  Testing("OOMPI_Request_array::Constructors");
  {
    MPI_Request mpi_request[10], *out;
    for (i = 0; i < 10; i++)
      mpi_request[i] = (MPI_Request) i;

    
    Testing("OOMPI_Request_array::MPI constructor");
    OOMPI_Request_array A(mpi_request, 10);
    out = A.Get_mpi();
    for (i = 0; i<10; i++)
      Midtest(out[i] == mpi_request[i]);
    Pass();

    Testing("OOMPI_Request_array::Copy constructor");
    OOMPI_Request_array B(A);
    for (i = 0; i < 10; i++)
      Midtest(B.Get_mpi()[i] == mpi_request[i]);
    Pass();

    Testing("OOMPI_Request_array::int constructor");
    OOMPI_Request_array C(4);
    for (i = 0; i < 4; i++)
      Midtest(C.Get_mpi()[i] == MPI_REQUEST_NULL);
    Pass();

    Testing("OOMPI_Request_array::operator=()");
    OOMPI_Request_array D;
    D = B;
    for (i = 0; i < 10; i++)
      Midtest(D.Get_mpi()[i] == mpi_request[i]);
    Pass();

    Pass(); // Constructors

    Testing("OOMPI_Request_array::operator[]");
    for (i = 0; i < 10; i++)
      Midtest (D[i].Get_mpi() == mpi_request[i]);
    Pass();
    
    Testing("OOMPI_Request_array::Get_size()");
    Midtest(B.Get_size() == 10);
    Midtest(C.Get_size() == 4);
    Midtest(D.Get_size() == 10);
    Pass();

    Testing("OOMPI_Request_array::operator==()");
    Test(B == D);

    Testing("OOMPI_Request_array::operator!=()");
    Test(!(B != D));
  }

  OOMPI_Request_array_test_wait();
  OOMPI_Request_array_test_test();
  OOMPI_Request_array_test_start();
 
  Pass(); // Request_array
}

void OOMPI_Request_array_test_wait() {
  
  int i, data[10], d, index;

  Testing("OOMPI_Request_array::Waitall()");
  {
    OOMPI_Request_array A(10);
    OOMPI_Status_array S(10), T;

    d = 0;
    A[0] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 200);
    A[1] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 201);
    A[2] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 202);
    A[3] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 203);
    A[4] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 204);
    A[5] = OOMPI_COMM_WORLD[to].Isend(data[d++], 200);
    A[6] = OOMPI_COMM_WORLD[to].Isend(data[d++], 201);
    A[7] = OOMPI_COMM_WORLD[to].Isend(data[d++], 202);
    A[8] = OOMPI_COMM_WORLD[to].Isend(data[d++], 203);
    A[9] = OOMPI_COMM_WORLD[to].Isend(data[d++], 204);

    S = A.Waitall();

    for (i = 0; i < 5; i++) 
      Midtest(S[i].Get_tag() == (200 + i), "Static status array");

    d = 0;
    A[0] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 200);
    A[1] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 201);
    A[2] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 202);
    A[3] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 203);
    A[4] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 204);
    A[5] = OOMPI_COMM_WORLD[to].Isend(data[d++], 200);
    A[6] = OOMPI_COMM_WORLD[to].Isend(data[d++], 201);
    A[7] = OOMPI_COMM_WORLD[to].Isend(data[d++], 202);
    A[8] = OOMPI_COMM_WORLD[to].Isend(data[d++], 203);
    A[9] = OOMPI_COMM_WORLD[to].Isend(data[d++], 204);
    
    A.Waitall(T);

    for (i = 0; i < 5; i++)
      Midtest(T[i].Get_tag() == (200 + i), "Expandable status array");
  }
  Pass();

  Testing("OOMPI_Request_array::Waitsome()");
  {
    OOMPI_Request_array A(7);
    OOMPI_Status_array S, T;
    int totalcount, outcount, indices[10];

    d = 0;
    A[0] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 200);
    A[1] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 201);
    A[2] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 202);
    A[3] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 203);
    A[4] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 204);
    A[5] = OOMPI_COMM_WORLD[to].Isend(data[d++], 200);
    A[6] = OOMPI_COMM_WORLD[to].Isend(data[d++], 201);

    outcount = 1;
    for (totalcount = 0; totalcount < 4; totalcount += outcount) {
      S = A.Waitsome(outcount, indices);
      for (i = 0; i < outcount; i++)
	if (indices[i] < 2)
          if (S[i].Get_tag() != (indices[i] + 200))
	    Fail("Status array return -- tag invalid");
    }
    Sync("Status array return");
    
    A.Set_size(10);
    A[7] = OOMPI_COMM_WORLD[to].Isend(data[d++], 202);
    A[8] = OOMPI_COMM_WORLD[to].Isend(data[d++], 203);
    A[9] = OOMPI_COMM_WORLD[to].Isend(data[d++], 204);

    S = A.Waitall();

    // Second test
    d = 0;
    A.Set_size(7);
    A[0] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 200);
    A[1] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 201);
    A[2] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 202);
    A[3] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 203);
    A[4] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 204);
    A[5] = OOMPI_COMM_WORLD[to].Isend(data[d++], 200);
    A[6] = OOMPI_COMM_WORLD[to].Isend(data[d++], 201);

    for (totalcount = 0; totalcount < 4; totalcount += outcount) {
      outcount = A.Waitsome(T, indices);
      for (i = 0; i < outcount; i++)
	if (indices[i] < 2)
          if (T[i].Get_tag() != (indices[i] + 200)) 
	    Fail("Status outcount return -- tag invalid");
    }
    Sync("Status outcount return");

    //d = 0;
    //Resetting d to zero here caused a freezeup on IBM MPI (possibly because
    //of the overlapping send and receive arrays that this caused).
    A.Set_size(10);
    A[7] = OOMPI_COMM_WORLD[to].Isend(data[d++], 202);
    A[8] = OOMPI_COMM_WORLD[to].Isend(data[d++], 203);
    A[9] = OOMPI_COMM_WORLD[to].Isend(data[d++], 204);

    A.Waitall();

    Pass();
  }

  Testing("OOMPI_Request_array::Waitany()");
  {
    OOMPI_Request_array A(6);
    OOMPI_Status S, T;
    
    d = 0;
    A[0] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 200);
    A[1] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 201);
    A[2] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 202);
    A[3] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 203);
    A[4] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 204);
    A[5] = OOMPI_COMM_WORLD[to].Isend(data[d++], 200);

    A[5].Wait();
    S = A.Waitany(index);
    Midtest(S.Get_tag() == 200 && index == 0, "Array status return");
    
    A.Set_size(10);
    A[6] = OOMPI_COMM_WORLD[to].Isend(data[d++], 201);
    A[7] = OOMPI_COMM_WORLD[to].Isend(data[d++], 202);
    A[8] = OOMPI_COMM_WORLD[to].Isend(data[d++], 203);
    A[9] = OOMPI_COMM_WORLD[to].Isend(data[d++], 204);

    A.Waitall();
    
    d = 0;
    A.Set_size(6);
    A[0] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 200);
    A[1] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 201);
    A[2] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 202);
    A[3] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 203);
    A[4] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 204);
    A[5] = OOMPI_COMM_WORLD[to].Isend(data[d++], 200);
    
    A[5].Wait();
    index = A.Waitany(T);
    Midtest(T.Get_tag() == 200 && index == 0, "Scalar return fail");
    
    A.Set_size(10);
    A[6] = OOMPI_COMM_WORLD[to].Isend(data[d++], 201);
    A[7] = OOMPI_COMM_WORLD[to].Isend(data[d++], 202);
    A[8] = OOMPI_COMM_WORLD[to].Isend(data[d++], 203);
    A[9] = OOMPI_COMM_WORLD[to].Isend(data[d++], 204);

    A.Waitall();
  }
  Pass();
}

void OOMPI_Request_array_test_test() {

  int i, data[10], d, index;
  bool flag;

  Testing("OOMPI_Request_array::Testall()");
  if (flags[SKIP_SGI20])
    Done("Skipped (SGI 2.0)");
  else
  {
    OOMPI_Request_array A(10);
    OOMPI_Status_array S(10), T;
#if 0
    bool done[10];
    for (i = 0; i < 10; i++)
      done[i] = false;
#endif

    d = 0;
    A[0] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 200);
    A[1] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 201);
    A[2] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 202);
    A[3] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 203);
    A[4] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 204);
    A[5] = OOMPI_COMM_WORLD[to].Isend(data[d++], 200);
    A[6] = OOMPI_COMM_WORLD[to].Isend(data[d++], 201);
    A[7] = OOMPI_COMM_WORLD[to].Isend(data[d++], 202);
    A[8] = OOMPI_COMM_WORLD[to].Isend(data[d++], 203);
    A[9] = OOMPI_COMM_WORLD[to].Isend(data[d++], 204);

#if 0
    int count = 0;
    do {
      S = A.Testall(flag);

      count++;
      for (i = 0; i < 5; i++)
	if (A[i].Get_mpi() == MPI_REQUEST_NULL && !done[i]) {
	  done[i] = true;
	  Midtest(S[i].Get_tag() == (200 + i), "Testall flag check");
	}
    } while(!flag);
    Sync();
#else
    do {
      S = A.Testall(flag);
    } while (!flag);

    for (i = 0; i < 5; i++)
      Midtest(S[i].Get_tag() == (200 + i), "Testall flag check");
#endif
    A.Waitall();

    d = 0;
    A[0] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 200);
    A[1] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 201);
    A[2] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 202);
    A[3] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 203);
    A[4] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 204);
    A[5] = OOMPI_COMM_WORLD[to].Isend(data[d++], 200);
    A[6] = OOMPI_COMM_WORLD[to].Isend(data[d++], 201);
    A[7] = OOMPI_COMM_WORLD[to].Isend(data[d++], 202);
    A[8] = OOMPI_COMM_WORLD[to].Isend(data[d++], 203);
    A[9] = OOMPI_COMM_WORLD[to].Isend(data[d++], 204);

    while (!A.Testall(T))
      continue;

    for (i = 0; i < 5; i++) 
      Midtest(T[i].Get_tag() == (200 + i), "Testall return value check");
    A.Waitall();

    Pass();
  }

  Testing("OOMPI_Request_array::Testsome()");
  {
    OOMPI_Request_array A(7);
    OOMPI_Status_array S, T;
    int outcount, totalcount, indices[10];

    d = 0;
    A[0] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 200);
    A[1] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 201);
    A[2] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 202);
    A[3] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 203);
    A[4] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 204);
    A[5] = OOMPI_COMM_WORLD[to].Isend(data[d++], 200);
    A[6] = OOMPI_COMM_WORLD[to].Isend(data[d++], 201);

    outcount = 1;
    for (totalcount = 0; totalcount < 4; totalcount += outcount) {
      S = A.Testsome(outcount, indices);
      for (i = 0; i < outcount; i++)
	if (indices[i] < 2)
          if (S[i].Get_tag() != ((indices[i] % 5) + 200))
	    Fail("Status array return -- tag invalid");
    }
    Sync();
 
    A.Set_size(10);
    A[7] = OOMPI_COMM_WORLD[to].Isend(data[d++], 202);
    A[8] = OOMPI_COMM_WORLD[to].Isend(data[d++], 203);
    A[9] = OOMPI_COMM_WORLD[to].Isend(data[d++], 204);

    A.Waitall();

    // Second test

    d = 0;
    A.Set_size(7);
    A[0] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 200);
    A[1] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 201);
    A[2] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 202);
    A[3] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 203);
    A[4] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 204);
    A[5] = OOMPI_COMM_WORLD[to].Isend(data[d++], 200);
    A[6] = OOMPI_COMM_WORLD[to].Isend(data[d++], 201);

    for (totalcount = 0; totalcount < 4; totalcount += outcount) {
      outcount = A.Testsome(T, indices);
      for (i = 0; i < outcount; i++)
	if (indices[i] < 2)
          if (T[i].Get_tag() != ((indices[i] % 5) + 200))
	    Fail("Status outcount return -- tag invalid");
    }
    Sync();

    A.Set_size(10);
    A[7] = OOMPI_COMM_WORLD[to].Isend(data[d++], 202);
    A[8] = OOMPI_COMM_WORLD[to].Isend(data[d++], 203);
    A[9] = OOMPI_COMM_WORLD[to].Isend(data[d++], 204);

    A.Waitall();

    Pass();
  }

  Testing("OOMPI_Request_array::Testany()");
  {
    OOMPI_Request_array A(6);
    OOMPI_Status S, T;
    
    d = 0;
    A[0] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 200);
    A[1] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 201);
    A[2] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 202);
    A[3] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 203);
    A[4] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 204);
    A[5] = OOMPI_COMM_WORLD[to].Isend(data[d++], 200);

    A[5].Wait();
    while (1) {
      S = A.Testany(index, flag);
      if (flag)
	break;
    }
    Midtest(S.Get_tag() == 200 && index == 0, "Array status return");
    
    A.Set_size(10);
    A[6] = OOMPI_COMM_WORLD[to].Isend(data[d++], 201);
    A[7] = OOMPI_COMM_WORLD[to].Isend(data[d++], 202);
    A[8] = OOMPI_COMM_WORLD[to].Isend(data[d++], 203);
    A[9] = OOMPI_COMM_WORLD[to].Isend(data[d++], 204);

    A.Waitall();
    
    A.Set_size(6);
    d = 0;
    A[0] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 200);
    A[1] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 201);
    A[2] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 202);
    A[3] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 203);
    A[4] = OOMPI_COMM_WORLD[from].Irecv(data[d++], 204);
    A[5] = OOMPI_COMM_WORLD[to].Isend(data[d++], 200);
    
    A[5].Wait();
    while (!A.Testany(T, index))
      continue;
    Midtest(T.Get_tag() == 200 && index == 0, "Scalar return fail");
    
    A.Set_size(10);
    A[6] = OOMPI_COMM_WORLD[to].Isend(data[d++], 201);
    A[7] = OOMPI_COMM_WORLD[to].Isend(data[d++], 202);
    A[8] = OOMPI_COMM_WORLD[to].Isend(data[d++], 203);
    A[9] = OOMPI_COMM_WORLD[to].Isend(data[d++], 204);

    A.Waitall();
  }
  Pass();
}

void OOMPI_Request_array_test_start() {
  
  int i, j, data[10], d;

  Testing("OOMPI_Request_array::Startall()");
  {
    OOMPI_Request_array A(10);
    OOMPI_Status_array S(10);

    d = 0;
    A[0] = OOMPI_COMM_WORLD[from].Recv_init(data[d++], 200);
    A[1] = OOMPI_COMM_WORLD[from].Recv_init(data[d++], 201);
    A[2] = OOMPI_COMM_WORLD[from].Recv_init(data[d++], 202);
    A[3] = OOMPI_COMM_WORLD[from].Recv_init(data[d++], 203);
    A[4] = OOMPI_COMM_WORLD[from].Recv_init(data[d++], 204);
    A[5] = OOMPI_COMM_WORLD[to].Send_init(data[d++], 200);
    A[6] = OOMPI_COMM_WORLD[to].Send_init(data[d++], 201);
    A[7] = OOMPI_COMM_WORLD[to].Send_init(data[d++], 202);
    A[8] = OOMPI_COMM_WORLD[to].Send_init(data[d++], 203);
    A[9] = OOMPI_COMM_WORLD[to].Send_init(data[d++], 204);
    
    for (i = 0; i < 1; i++) {
      A.Startall();
      S = A.Waitall();
	
      for (j = 0; j < 5; j++) {
	Midtest(S[j].Get_tag() == 200 + j);
      }
    }
    A.Freeall();
  }
  Pass();


  OOMPI_Request_array B(5);

  Testing("OOMPI_Request_array::Set_size()");
  B.Set_size(10);
  Midtest(B.Get_size() == 10);
  B.Set_size(3); 
  Midtest(B.Get_size() == 3);
  B.Set_size(7);
  Midtest(B.Get_size() == 7);
  Pass();
}

