// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: request_test.cc,v 1.5 2003/02/13 21:07:48 jsquyres Exp $
//
// OOMPI test program
// Test OOMPI_Request functionality
//

#include "oompi.h"
#include "oompi_test.h"


//
// OOMPI_Request_test
//
void
OOMPI_Request_test()
{

  Testing("OOMPI_Request");

  static OOMPI_Request req1, req2;
  OOMPI_Status stat1, stat2;
  int flag1, flag2;
  int message;

  Testing("OOMPI_Request::Constructors");
  {
    Testing("OOMPI_Request::MPI constructor");
    OOMPI_Request B(MPI_REQUEST_NULL);
    Test(B.Get_mpi() == MPI_REQUEST_NULL);
    
    Testing("OOMPI_Request::Copy constructor");
    OOMPI_Request C(B);
    Test(B.Get_mpi() == C.Get_mpi());
    
    Testing("OOMPI_Request::operator=()");
    OOMPI_Request D;
    D = B;
    Test(B.Get_mpi() == D.Get_mpi());

    Pass();

    Testing("OOMPI_Request::Is_null()");
    Test(B.Is_null());

    Testing("OOMPI_Request::operator==()");
    Test(B == C);

    Testing("OOMPI_Request::operator!=()");
    Test(!(B != C));
  }


  Testing("OOMPI_Request::Wait()");

  req1 = OOMPI_COMM_WORLD[to].Isend(my_rank, 201);
  req2 = OOMPI_COMM_WORLD[from].Irecv(message, 201);
  stat1 = req1.Wait();
  stat2 = req2.Wait();

  Test(stat2.Get_source() == from &&
       stat2.Get_count(OOMPI_INT) == 1 &&
       stat2.Get_tag() == 201 &&
       message == from);

  Testing("OOMPI_Request::Test()");

  message = 0;
  req1 = OOMPI_COMM_WORLD[to].Isend(my_rank, 201);
  req2 = OOMPI_COMM_WORLD[from].Irecv(message, 201);
  flag1 = flag2 = 0;
  while (flag1 + flag2 != 2) {
    if (flag1 == 0)
      flag1 = req1.Test(stat1);
    if (flag2 == 0)
      flag2 = req2.Test(stat2);
  }
  
  Test(stat2.Get_source() == from &&
       stat2.Get_count(MPI_INT) == 1 &&
       stat2.Get_tag() == 201 &&
       message == from);

  Testing("OOMPI_Request::Start()");
  {
    if (my_rank %2 == 1) {
      int num = 333;
      OOMPI_Request A(OOMPI_COMM_WORLD[to].Send_init(num,201));
      for (int i=0; i < 10; i++) {
	num = i * 111;
	A.Start();
	A.Wait();
      }
      A.Free();
    }
    else if (my_rank %2 == 0)  {
      int num;
      OOMPI_Request B(OOMPI_COMM_WORLD[from].Recv_init(num,201));
      OOMPI_Status S;
      
      for (int i=0; i<10; i++) {
	B.Start();
	S = B.Wait();
	
	if (!(S.Get_source() == from &&
	      S.Get_count(MPI_INT) == 1 &&
	      S.Get_tag() == 201 &&
	      num == i*111)) {
	  Fail();
	}
      }
      B.Free();
    }
    Pass();
  }
  Pass(); // Request
}
