// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: status_array_test.cc,v 1.5 2003/02/13 21:07:48 jsquyres Exp $
//
// OOMPI test program
// Test OOMPI_Status_array functionality
//

#include "oompi.h"
#include "oompi_test.h"

void get_my_status(OOMPI_Status &a, int i);


//
// OOMPI_Status_array_test
//
void
OOMPI_Status_array_test()
{
  Testing("OOMPI_Status_array");
  Testing("OOMPI_Status_array::operator[]()");
  OOMPI_Status_array B(5);
  
  Testing("OOMPI_Status_array::Constructors");
  {
    int i;
    MPI_Status mpi_status[10], *out;

    for (i=0; i<10; i++)
      mpi_status[i].MPI_SOURCE = i;

    
    Testing("OOMPI_Status_array::MPI constructor");
    OOMPI_Status_array A(mpi_status, 10);
    out = A.Get_mpi();
    for (i = 0; i<10; i++)
      Midtest(out[i].MPI_SOURCE == mpi_status[i].MPI_SOURCE);
    Pass();

    Testing("OOMPI_Status_array::Copy constructor");
    OOMPI_Status_array BB(A);
	out = BB.Get_mpi();
    for (i=0; i<10; i++)
      Midtest(out[i].MPI_SOURCE == mpi_status[i].MPI_SOURCE);
    Pass();

    Testing("OOMPI_Status_array::int constructor");
    OOMPI_Status_array C(4);
    Pass();

    Testing("OOMPI_Status_array::operator=()");
    OOMPI_Status_array D;
    D = BB;
	out = D.Get_mpi();
    for (i=0; i<10; i++)
      Midtest(out[i].MPI_SOURCE == mpi_status[i].MPI_SOURCE);
    Pass();

    Pass(); // Constructors
  }

  if (my_rank == 0) {
    int num, i;
    for (i=0; i<5; i++) {
      B[i] = OOMPI_COMM_WORLD[1].Recv(num, i);
    }
    
    int pass = 1;
    for (i=0; i<5; i++) {
      if (B[i].Get_source() != 1)
	pass = 0;
      if (B[i].Get_tag() != i)
	pass = 0;
    }
    
    for (i=0; i<5; i++) {
      get_my_status(B[i],i);
    }

    for (i=0; i<5; i++) {
      if (B[i].Get_source() != 1)
	pass = 0;
      if (B[i].Get_tag() != i*2)
	pass = 0;
    }

    if (pass == 0)
      Fail();

  }
  if (my_rank == 1) {
    int i, num;
    for (i=0; i<5; i++) {
      num = i * 111;
      OOMPI_COMM_WORLD[0].Send(num, i);
    }
    for (i=0; i<5; i++) {
      num = i * 111;
      OOMPI_COMM_WORLD[0].Send(num, i*2);
    }
  }
  Pass();

  Testing("OOMPI_Status_array::Get_mpi");
  if (my_rank == 0) {
    // B will not be initialized except on rank 0, so always pass on other
    // ranks
    int i, pass;
    MPI_Status *a;
    a = B.Get_mpi();
    
    pass = 1;
    for (i=0; i<5; i++) {
      if (B[i].Get_source() != a[i].MPI_SOURCE)
	pass = 0;
      if (B[i].Get_tag() != a[i].MPI_TAG)
	pass = 0;
    }
    if (pass == 0)
      Fail();
  }
  Pass();

  Testing("OOMPI_Status_array::Get_size");
  OOMPI_Status_array X(10);
  Test(X.Get_size() == 10);

  Testing("OOMPI_Status_array::Set_size");
  B.Set_size(10);
  if (B.Get_size() != 10)
    Fail();
  B.Set_size(3); 
  if (B.Get_size() != 3)
    Fail();
  B.Set_size(7);
  if (B.Get_size() != 7)
    Fail();
  Pass();

  Pass();
}


void 
get_my_status(OOMPI_Status &a, int i)
{
  int num;
  a = OOMPI_COMM_WORLD[1].Recv(num, i*2);
}

