// -*- c++ -*-
//
// Copyright (c) 2002-2003 Indiana University.  All rights reserved.
// Copyright (c) 1996, 1997, 1998, 2000 University of Notre Dame.
//                         All rights reserved.
// 
// This file is part of the OOMPI software package.  For license
// information, see the LICENSE file in the top level directory of the
// OOMPI source distribution.
//
// $Id: status_test.cc,v 1.6 2003/02/13 21:07:48 jsquyres Exp $
//
// OOMPI test program
// Test OOMPI_Status functionality
//

#include "oompi.h"
#include "oompi_test.h"


//
// OOMPI_Status_test
//
void
OOMPI_Status_test()
{
  Testing("OOMPI_Status");

  Testing("OOMPI_Status::Constructors"); 
  {
    MPI_Status A;
    A.MPI_SOURCE = 1;
    A.MPI_TAG = 2;

    Testing("OOMPI_Status::MPI constructor");
    OOMPI_Status B(A);
    Test(B.Get_source() == A.MPI_SOURCE);
    
    Testing("OOMPI_Status::Copy constructor");
    OOMPI_Status C(B);
    Test(B.Get_source() == C.Get_source());
    
    Testing("OOMPI_Status::operator=");
    OOMPI_Status D;
    D = B;
    Test(B.Get_source() == D.Get_source());

    Pass();
  }

  {
    MPI_Status B;
    int numA, numB;
    OOMPI_Status A;
    OOMPI_Request_array req(2);
    int num, count, element;

    num = 555;
    req[0] = OOMPI_COMM_WORLD[to].Isend(num, 201);
    req[1] = OOMPI_COMM_WORLD[to].Isend(num, 201);
    A = OOMPI_COMM_WORLD[from].Recv(numA,201);
    MPI_Recv(&numB, 1, MPI_INT, from, 201, MPI_COMM_WORLD, &B);
    req.Waitall();

    Testing("OOMPI_Status::Get_source()");
    Test(A.Get_source() == B.MPI_SOURCE);

    Testing("OOMPI_Status::Get_tag()");
    Test(A.Get_tag() == B.MPI_TAG);

    Testing("OOMPI_Status::Get_count()");
    MPI_Get_count(&B, MPI_INT, &count);
    Test(A.Get_count(MPI_INT) == count);
    
    Testing("OOMPI_Status::Get_error()");
    Done("Cannot test");

    Testing("OOMPI_Status::Get_element()");
    MPI_Get_elements(&B, MPI_INT, &element);
    Test(A.Get_elements(OOMPI_INT) == element);
  }

  Testing("OOMPI_Status::Test_cancelled()");
  if (flags[SKIP_SGI20])
    Done("Skipped (SGI 2.0)");
  else {
    OOMPI_Request request;
    OOMPI_Status status;
    int i;
    
    request = OOMPI_COMM_WORLD[from].Recv_init(i);
    request.Start();
    request.Cancel();
    status = request.Wait();
    request.Free();
    
    Test(status.Test_cancelled());
  }

  Testing("OOMPI_Status::OOMPI_Intra_comm");
  {
    Testing("OOMPI_Intra_comm::Probe()");
    {
      int i;
      OOMPI_Request req;
      OOMPI_Status stat;

      i = my_rank;
      req = OOMPI_COMM_WORLD[to].Isend(my_rank, 201);
      stat = OOMPI_COMM_WORLD.Probe(201);
      OOMPI_COMM_WORLD.Recv(i, 201);
      req.Wait();

      Midtest(stat.Get_tag() == 201 && stat.Get_source() == from);
    }
    Pass();

    Testing("OOMPI_Intra_comm::Iprobe()");
    {
      int i;
      OOMPI_Request req;
      OOMPI_Status stat;
      bool flag;

      i = my_rank;
      OOMPI_COMM_WORLD.Iprobe(201, flag);
      Midtest(!flag && !OOMPI_COMM_WORLD.Iprobe(201), "Pre-send Iprobe fail");

      req = OOMPI_COMM_WORLD[to].Isend(my_rank, 201);
      MPI_Barrier(MPI_COMM_WORLD);
      stat = OOMPI_COMM_WORLD.Iprobe(201, flag);
      OOMPI_COMM_WORLD.Recv(i, 201);
      req.Wait();

      Midtest(flag && stat.Get_tag() == 201 && stat.Get_source() == from,
	      "Post-send Iprobe fail");
    }
    Pass();
  }
  Pass(); // OOMPI_Intra_comm

  Pass(); // OOMPI_Status
}
